@@ -10,32 +10,30 @@ without even the IMPLIED WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE. See the terms and conditions of the GNU Lesser General Public License
 for more details.  You should have received a copy of the GNU Lesser General
 Public License along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifndef GOTCHA_AUXV_H
 #define GOTCHA_AUXV_H
 
 #include <elf.h>
+#include <errno.h>
+#include <fcntl.h>
 #include <link.h>
 #include <stdio.h>
-#include <fcntl.h>
-#include <string.h>
-#include <errno.h>
-#include <unistd.h>
 #include <stdlib.h>
-#include <sys/types.h>
+#include <string.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
 
-int is_vdso(struct link_map *map);
+int is_vdso(const struct link_map *map);
 unsigned int get_auxv_pagesize();
 
-//Do not use, exposed only for unit testing
+// Do not use, exposed only for unit testing
 int parse_auxv_contents();
 struct link_map *get_vdso_from_auxv();
 struct link_map *get_vdso_from_aliases();
 struct link_map *get_vdso_from_maps();
 
-
-
 #endif
