@@ -10,7 +10,7 @@ without even the IMPLIED WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE. See the terms and conditions of the GNU Lesser General Public License
 for more details.  You should have received a copy of the GNU Lesser General
 Public License along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 // TODO: Determine whether this interface should stay on in this form
@@ -18,15 +18,12 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #ifndef GOTCHA_LIBRARY_FILTERS_H
 #define GOTCHA_LIBRARY_FILTERS_H
 #include <link.h>
+
 #include "gotcha_utils.h"
 
-int alwaysTrue(struct link_map* candidate KNOWN_UNUSED);
-extern int (*libraryFilterFunc)(struct link_map*);
+int alwaysTrue(struct link_map *candidate KNOWN_UNUSED);
+extern int (*libraryFilterFunc)(struct link_map *);
 
-int trueIfNameMatches(struct link_map* target);
-int trueIfLast(struct link_map* target);
-void filterLibrariesByName(const char* nameFilter);
-void onlyFilterLast();
-void setLibraryFilterFunc(int(*new_func)(struct link_map*));
-void restoreLibraryFilterFunc();
+int trueIfNameMatches(struct link_map *target);
+int trueIfLast(struct link_map *target);
 #endif
