@@ -6,13 +6,13 @@
  *      Description:  Implementation a interface library to hook into applications
  *                    using the GOTCHA library
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
- *      Author:   Thoams Roehl (tr), thomas.roehl@gmail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,6 +38,7 @@
 #include <pthread.h>
 #include <unistd.h>
 #include <time.h>
+#include <bstrlib.h>
 
 #include <likwid.h>
 #include <error.h>
@@ -54,7 +55,7 @@ static FILE* output_file = NULL;
 static int stopIssued = 0;
 static pthread_mutex_t stopMutex;
 
-int appdaemon_register_exit(appdaemon_exit_func f)
+static int appdaemon_register_exit(appdaemon_exit_func f)
 {
     if (appdaemon_num_exit_funcs < APPDAEMON_MAX_EXIT_FUNCS)
     {
@@ -111,7 +112,7 @@ static int parse_gpustr(char* gpuStr, int* numGpus, int** gpuIds)
 {
     // Create bstring
     bstring bGpuStr = bfromcstr(gpuStr);
-    
+    int (*ownatoi)(const char*) = atoi;
     // Parse list
     struct bstrList* gpuTokens = bsplit(bGpuStr,',');
     int tmpNumGpus = gpuTokens->qty;
@@ -129,7 +130,7 @@ static int parse_gpustr(char* gpuStr, int* numGpus, int** gpuIds)
     // Parse ids to int
     for (int i = 0; i < tmpNumGpus; i++)
     {
-        tmpGpuIds[i] = atoi(bdata(gpuTokens->entry[i]));
+        tmpGpuIds[i] = ownatoi(bdata(gpuTokens->entry[i]));
     }
 
     // Copy data
@@ -515,6 +516,7 @@ static void* appdaemon_timeline_main(void* arg)
 /*
 Main
 */
+__attribute__((visibility("default")))
 int __libc_start_main(int (*main) (int,char **,char **),
               int argc,char **ubp_av,
               void (*init) (void),
