@@ -5,14 +5,14 @@
  *
  *      Description:  Implementation of affinity module.
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -177,76 +177,219 @@ static int get_id_of_type(hwloc_obj_t base, hwloc_obj_type_t type)
     return -1;
 }
 
+#define AFF_FREE_AND_RESET(ptr) if (ptr != NULL) { free(ptr); ptr = NULL; }
+
 static int create_lookups()
 {
+    int err = 0;
     int do_cache = 1;
     int cachelimit = 0;
     int cacheIdx = -1;
-    topology_init();
-    numa_init();
+    int * tmp = NULL;
+    int num_tmp = 0;
+    err = topology_init();
+    if (err != 0)
+    {
+        return err;
+    }
+    err = numa_init();
+    if (err != 0)
+    {
+        return err;
+    }
     CpuTopology_t cputopo = get_cpuTopology();
     NumaTopology_t ntopo = get_numaTopology();
     if (!affinity_thread2core_lookup)
     {
         affinity_thread2core_lookup = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!affinity_thread2core_lookup)
+        {
+            return -ENOMEM;
+        }
         memset(affinity_thread2core_lookup, -1, cputopo->numHWThreads*sizeof(int));
     }
     if (!affinity_thread2socket_lookup)
     {
         affinity_thread2socket_lookup = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!affinity_thread2socket_lookup)
+        {
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(affinity_thread2socket_lookup, -1, cputopo->numHWThreads*sizeof(int));
     }
     if (!affinity_thread2sharedl3_lookup)
     {
         affinity_thread2sharedl3_lookup = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!affinity_thread2sharedl3_lookup)
+        {
+            AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(affinity_thread2sharedl3_lookup, -1, cputopo->numHWThreads*sizeof(int));
     }
     if (!affinity_thread2numa_lookup)
     {
         affinity_thread2numa_lookup = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!affinity_thread2numa_lookup)
+        {
+            AFF_FREE_AND_RESET(affinity_thread2sharedl3_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(affinity_thread2numa_lookup, -1, cputopo->numHWThreads*sizeof(int));
     }
     if (!affinity_thread2die_lookup)
     {
         affinity_thread2die_lookup = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!affinity_thread2die_lookup)
+        {
+            AFF_FREE_AND_RESET(affinity_thread2numa_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2sharedl3_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(affinity_thread2die_lookup, -1, cputopo->numHWThreads*sizeof(int));
     }
     if (!socket_lock)
     {
         socket_lock = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!socket_lock)
+        {
+            AFF_FREE_AND_RESET(affinity_thread2die_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2numa_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2sharedl3_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(socket_lock, LOCK_INIT, cputopo->numHWThreads*sizeof(int));
     }
     if (!die_lock)
     {
         die_lock = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!die_lock)
+        {
+            AFF_FREE_AND_RESET(socket_lock);
+            AFF_FREE_AND_RESET(affinity_thread2die_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2numa_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2sharedl3_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(die_lock, LOCK_INIT, cputopo->numHWThreads*sizeof(int));
     }
     if (!numa_lock)
     {
         numa_lock = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!numa_lock)
+        {
+            AFF_FREE_AND_RESET(die_lock);
+            AFF_FREE_AND_RESET(socket_lock);
+            AFF_FREE_AND_RESET(affinity_thread2die_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2numa_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2sharedl3_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(numa_lock, LOCK_INIT, cputopo->numHWThreads*sizeof(int));
     }
     if (!core_lock)
     {
         core_lock = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!core_lock)
+        {
+            AFF_FREE_AND_RESET(numa_lock);
+            AFF_FREE_AND_RESET(die_lock);
+            AFF_FREE_AND_RESET(socket_lock);
+            AFF_FREE_AND_RESET(affinity_thread2die_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2numa_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2sharedl3_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(core_lock, LOCK_INIT, cputopo->numHWThreads*sizeof(int));
     }
     if (!tile_lock)
     {
         tile_lock = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!tile_lock)
+        {
+            AFF_FREE_AND_RESET(core_lock);
+            AFF_FREE_AND_RESET(numa_lock);
+            AFF_FREE_AND_RESET(die_lock);
+            AFF_FREE_AND_RESET(socket_lock);
+            AFF_FREE_AND_RESET(affinity_thread2die_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2numa_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2sharedl3_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(tile_lock, LOCK_INIT, cputopo->numHWThreads*sizeof(int));
     }
     if (!sharedl2_lock)
     {
         sharedl2_lock = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!sharedl2_lock)
+        {
+            AFF_FREE_AND_RESET(tile_lock);
+            AFF_FREE_AND_RESET(core_lock);
+            AFF_FREE_AND_RESET(numa_lock);
+            AFF_FREE_AND_RESET(die_lock);
+            AFF_FREE_AND_RESET(socket_lock);
+            AFF_FREE_AND_RESET(affinity_thread2die_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2numa_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2sharedl3_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(sharedl2_lock, LOCK_INIT, cputopo->numHWThreads*sizeof(int));
     }
     if (!sharedl3_lock)
     {
         sharedl3_lock = malloc(cputopo->numHWThreads * sizeof(int));
+        if (!sharedl3_lock)
+        {
+            AFF_FREE_AND_RESET(sharedl2_lock);
+            AFF_FREE_AND_RESET(tile_lock);
+            AFF_FREE_AND_RESET(core_lock);
+            AFF_FREE_AND_RESET(numa_lock);
+            AFF_FREE_AND_RESET(die_lock);
+            AFF_FREE_AND_RESET(socket_lock);
+            AFF_FREE_AND_RESET(affinity_thread2die_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2numa_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2sharedl3_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+            AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+            return -ENOMEM;
+        }
         memset(sharedl3_lock, LOCK_INIT, cputopo->numHWThreads*sizeof(int));
     }
-
+    tmp = malloc(cputopo->numHWThreads * sizeof(int));
+    if (!tmp)
+    {
+        AFF_FREE_AND_RESET(sharedl3_lock);
+        AFF_FREE_AND_RESET(sharedl2_lock);
+        AFF_FREE_AND_RESET(tile_lock);
+        AFF_FREE_AND_RESET(core_lock);
+        AFF_FREE_AND_RESET(numa_lock);
+        AFF_FREE_AND_RESET(die_lock);
+        AFF_FREE_AND_RESET(socket_lock);
+        AFF_FREE_AND_RESET(affinity_thread2die_lookup);
+        AFF_FREE_AND_RESET(affinity_thread2numa_lookup);
+        AFF_FREE_AND_RESET(affinity_thread2sharedl3_lookup);
+        AFF_FREE_AND_RESET(affinity_thread2socket_lookup);
+        AFF_FREE_AND_RESET(affinity_thread2core_lookup);
+        return -ENOMEM;
+    }
 
     int num_pu = cputopo->numHWThreads;
     if (cputopo->numCacheLevels == 0)
@@ -259,6 +402,23 @@ static int create_lookups()
         cacheIdx = -1;
     }
     for (int pu_idx = 0; pu_idx < num_pu; pu_idx++)
+    {
+        HWThread* t = &cputopo->threadPool[pu_idx];
+        int found = 0;
+        for (int j = 0; j < num_tmp; j++)
+        {
+            if (tmp[j] == t->packageId)
+            {
+                found = 1;
+                break;
+            }
+        }
+        if (!found)
+        {
+            tmp[num_tmp++] = t->packageId;
+        }
+    }
+    for (int pu_idx = 0; pu_idx < num_pu; pu_idx++)
     {
         HWThread* t = &cputopo->threadPool[pu_idx];
         int hwthreadid = t->apicId;
@@ -268,6 +428,17 @@ static int create_lookups()
         int dies_per_socket = MAX(cputopo->numDies/cputopo->numSockets, 1);
         affinity_thread2core_lookup[hwthreadid] = coreid;
         affinity_thread2socket_lookup[hwthreadid] = sockid;
+        for (int j = 0; j < num_tmp; j++)
+        {
+            if (affinity_thread2socket_lookup[hwthreadid] == tmp[j])
+            {
+                if (affinity_thread2socket_lookup[hwthreadid] != j)
+                {
+                    affinity_thread2socket_lookup[hwthreadid] = j;
+                    sockid = j;
+                }
+            }
+        }
         affinity_thread2die_lookup[hwthreadid] = (sockid * dies_per_socket) + dieid;
         int memid = 0;
         for (int n = 0; n < ntopo->numberOfNodes; n++)
@@ -294,7 +465,7 @@ static int create_lookups()
                                         affinity_thread2sharedl3_lookup[hwthreadid],
                                         affinity_thread2numa_lookup[hwthreadid]);
     }
-
+    free(tmp);
     return 0;
 }
 
@@ -352,6 +523,12 @@ static int affinity_addNodeDomain(AffinityDomain* domain, int* help)
         {
             return -ENOMEM;
         }
+        domain->tag = malloc(2);
+        if (!domain->tag)
+        {
+            free(domain->processorList);
+            return -ENOMEM;
+        }
         int offset = 0;
         int cores = 0;
         for (int i = 0; i < MAX(cputopo->numSockets, 1); i++)
@@ -366,7 +543,8 @@ static int affinity_addNodeDomain(AffinityDomain* domain, int* help)
         }
         domain->numberOfProcessors = offset;
         domain->numberOfCores = cores;
-        domain->tag = bformat("N");
+        domain->tag[0] = 'N';
+        domain->tag[1] = '\0';
         DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain N: %d HW threads on %d cores, domain->numberOfProcessors, domain->numberOfCores);
         return 0;
     }
@@ -388,6 +566,12 @@ static int affinity_addSocketDomain(int socket, AffinityDomain* domain, int* hel
         {
             return -ENOMEM;
         }
+        domain->tag = malloc(10);
+        if (!domain->tag)
+        {
+            free(domain->processorList);
+            return -ENOMEM;
+        }
         int tmp = treeFillNextEntries(cputopo->topologyTree,
                                       domain->processorList,
                                       0,
@@ -397,7 +581,7 @@ static int affinity_addSocketDomain(int socket, AffinityDomain* domain, int* hel
         tmp = MIN(tmp, domain->numberOfProcessors);
         domain->numberOfProcessors = tmp;
         domain->numberOfCores = affinity_countSocketCores(domain->numberOfProcessors, domain->processorList, help);
-        domain->tag = bformat("S%d", socket);
+        snprintf(domain->tag, 9, "S%d", socket);
         DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain S%d: %d HW threads on %d cores, socket, domain->numberOfProcessors, domain->numberOfCores);
         return 0;
     }
@@ -423,6 +607,12 @@ static int affinity_addDieDomain(int socket, int die, AffinityDomain* domain, in
         {
             return -ENOMEM;
         }
+        domain->tag = malloc(10);
+        if (!domain->tag)
+        {
+            free(domain->processorList);
+            return -ENOMEM;
+        }
         domain->numberOfProcessors = numThreadsPerDie;
         domain->numberOfCores = numCoresPerDie;
         int tmp = treeFillNextEntries(cputopo->topologyTree,
@@ -434,7 +624,7 @@ static int affinity_addDieDomain(int socket, int die, AffinityDomain* domain, in
                                       domain->numberOfProcessors);
         domain->numberOfProcessors = tmp;
         domain->numberOfCores = affinity_countSocketCores(domain->numberOfProcessors, domain->processorList, help);
-        domain->tag = bformat("D%d", dieId);
+        snprintf(domain->tag, 9, "D%d", dieId);
         DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain D%d: %d HW threads on %d cores, dieId, domain->numberOfProcessors, domain->numberOfCores);
         return 0;
     }
@@ -459,6 +649,12 @@ static int affinity_addCacheDomain(int socket, int cacheId, AffinityDomain* doma
         {
             return -ENOMEM;
         }
+        domain->tag = malloc(10);
+        if (!domain->tag)
+        {
+            free(domain->processorList);
+            return -ENOMEM;
+        }
         domain->numberOfProcessors = numThreadsPerCache;
         domain->numberOfCores = numCoresPerCache;
         int tmp = treeFillNextEntries(cputopo->topologyTree,
@@ -470,18 +666,19 @@ static int affinity_addCacheDomain(int socket, int cacheId, AffinityDomain* doma
                                       domain->numberOfProcessors);
         domain->numberOfProcessors = tmp;
         domain->numberOfCores = affinity_countSocketCores(domain->numberOfProcessors, domain->processorList, help);
-        domain->tag = bformat("C%d", cid);
+        snprintf(domain->tag, 9, "C%d", cid);
         DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain C%d: %d HW threads on %d cores, cid, domain->numberOfProcessors, domain->numberOfCores);
         return 0;
     }
     return -EINVAL;
 }
 
-static int affinity_addMemoryDomain(int nodeId, AffinityDomain* domain, int* help)
+static int _affinity_addMemoryDomain(int nodeId, AffinityDomain* domain, int* help)
 {
     CpuTopology_t cputopo = get_cpuTopology();
     NumaTopology_t numatopo = get_numaTopology();
-    if (!domain)
+    int num_hwthreads = 0;
+    if ((nodeId < 0) || (!domain))
     {
         return -EINVAL;
     }
@@ -494,42 +691,163 @@ static int affinity_addMemoryDomain(int nodeId, AffinityDomain* domain, int* hel
             {
                 return -ENOMEM;
             }
+            domain->tag = malloc(10);
+            if (!domain->tag)
+            {
+                free(domain->processorList);
+                return -ENOMEM;
+            }
             for (int i = 0; i < numatopo->nodes[nodeId].numberOfProcessors; i++)
             {
-                domain->processorList[i] = (int)numatopo->nodes[nodeId].processors[i];
+                for (int j = 0; j < cputopo->numHWThreads; j++)
+                {
+                    if (cputopo->threadPool[j].apicId == numatopo->nodes[nodeId].processors[i] && cputopo->threadPool[j].inCpuSet == 1)
+                    {
+                        domain->processorList[num_hwthreads++] = (int)numatopo->nodes[nodeId].processors[i];
+                    }
+                }
             }
-            domain->numberOfProcessors = numatopo->nodes[nodeId].numberOfProcessors;
+            domain->numberOfProcessors = num_hwthreads;
             domain->numberOfCores = affinity_countSocketCores(domain->numberOfProcessors, domain->processorList, help);
-            domain->tag = bformat("M%d", nodeId);
-            DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain M%d: %d HW threads on %d cores, nodeId, domain->numberOfProcessors, domain->numberOfCores);
+            snprintf(domain->tag, 9, "M%d", nodeId);
             return 0;
         }
     }
     return -EINVAL;
 }
 
+static int affinity_addMemoryDomain(int nodeId, AffinityDomain* domain, int* help)
+{
+    int err = _affinity_addMemoryDomain(nodeId, domain, help);
+    if (err == 0)
+    {
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain M%d: %d HW threads on %d cores, nodeId, domain->numberOfProcessors, domain->numberOfCores);
+    }
+    return err;
+}
+
+#ifdef LIKWID_WITH_NVMON
+static int affinity_addCudaDomain(int nodeId, AffinityDomain* domain, int offset, int* help)
+{
+    int err = 0;
+    CpuTopology_t cputopo = get_cpuTopology();
+    NumaTopology_t numatopo = get_numaTopology();
+    CudaTopology_t cudatopo = get_cudaTopology();
+
+    if ((nodeId < 0) || (!domain))
+    {
+        return -EINVAL;
+    }
+    if (cputopo && numatopo && cudatopo)
+    {
+        if (nodeId >= 0 && nodeId < cudatopo->numDevices)
+        {
+            CudaDevice* cudadev = &cudatopo->devices[nodeId];
+            if (cudadev->numaNode >= 0)
+            {
+                err = _affinity_addMemoryDomain(cudadev->numaNode, domain, help);
+                if (err == 0)
+                {
+                    snprintf(domain->tag, 9, "G%d", nodeId+offset);
+                    DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain %s: %d HW threads on %d cores, domain->tag, domain->numberOfProcessors, domain->numberOfCores);
+                    return 0;
+                }
+            }
+        }
+    }
+    return -EINVAL;
+}
+#endif
+
+#ifdef LIKWID_WITH_ROCMON
+static int affinity_addRocmDomain(int nodeId, AffinityDomain* domain, int offset, int* help)
+{
+    int err = 0;
+    CpuTopology_t cputopo = get_cpuTopology();
+    NumaTopology_t numatopo = get_numaTopology();
+    RocmTopology_t rocmtopo = get_rocmTopology();
+
+    if ((nodeId < 0) || (!domain))
+    {
+        return -EINVAL;
+    }
+    if (cputopo && numatopo && rocmtopo)
+    {
+        if (nodeId >= 0 && nodeId < rocmtopo->numDevices)
+        {
+            RocmDevice* rocmdev = &rocmtopo->devices[nodeId];
+            if (rocmdev->numaNode >= 0)
+            {
+                err = _affinity_addMemoryDomain(rocmdev->numaNode, domain, help);
+                if (err == 0)
+                {
+                    snprintf(domain->tag, 9, "G%d", nodeId+offset);
+                    DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain %s: %d HW threads on %d cores, domain->tag, domain->numberOfProcessors, domain->numberOfCores);
+                    return 0;
+                }
+            }
+            else
+            {
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, Skipping affinity domain G%d because NUMA node unknown, nodeId);
+            }
+        }
+    }
+    return -EINVAL;
+}
+#endif
+
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
-void
-affinity_init()
+int
+affinity_init(void)
 {
+    int err = 0;
     int numberOfDomains = 1; /* all systems have the node domain */
     int finalNumberOfDomains = 0;
     int currentDomain;
     int subCounter = 0;
     int offset = 0;
     int domid = 0;
-    int tmp;
+    int tmp = 0;
     if (affinity_initialized == 1)
     {
-        return;
+        return 0;
+    }
+    err = topology_init();
+    if (err != 0)
+    {
+        return err;
     }
-    topology_init();
     CpuTopology_t cputopo = get_cpuTopology();
     CpuInfo_t cpuinfo = get_cpuInfo();
-    numa_init();
+    err = numa_init();
+    if (err != 0)
+    {
+        return err;
+    }
     NumaTopology_t numatopo = get_numaTopology();
 
+#ifdef LIKWID_WITH_NVMON
+    int doCudaDomains = 1;
+    int numCudaDomains = 0;
+    CudaTopology_t cudatopo = NULL;
+    err = topology_cuda_init();
+    if (err != 0)
+    {
+        doCudaDomains = 0;
+    }
+#endif
+#ifdef LIKWID_WITH_ROCMON
+    int doRocmDomains = 1;
+    int numRocmDomains = 0;
+    RocmTopology_t rocmtopo = NULL;
+    err = topology_rocm_init();
+    if (err != 0)
+    {
+        doRocmDomains = 0;
+    }
+#endif
+
     int doCacheDomains = 1;
     int numberOfCacheDomains = 0;
     int numberOfCoresPerCache = 0;
@@ -553,29 +871,66 @@ affinity_init()
         numberOfCoresPerCache = numberOfProcessorsPerCache / cputopo->numThreadsPerCore;
         DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity: CPU cores per LLC %d, numberOfCoresPerCache);
         int numCachesPerSocket = cputopo->numCoresPerSocket / numberOfCoresPerCache;
-        numberOfCacheDomains = cputopo->numSockets * numCachesPerSocket;
+        numberOfCacheDomains = cputopo->numSockets * MAX(numCachesPerSocket, 1);
         DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity: Cache domains %d, numberOfCacheDomains);
         numberOfDomains += numberOfCacheDomains;
     }
     numberOfDomains += numatopo->numberOfNodes;
     DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity: NUMA domains %d, numatopo->numberOfNodes);
+#if defined(LIKWID_WITH_NVMON) || defined(LIKWID_WITH_ROCMON)
+    int gpuDomains = 0;
+#ifdef LIKWID_WITH_NVMON
+    if (doCudaDomains)
+    {
+        cudatopo = get_cudaTopology();
+        for (int i = 0; i < cudatopo->numDevices; i++)
+        {
+            CudaDevice* dev = &cudatopo->devices[i];
+            if (dev->numaNode >= 0)
+            {
+                numCudaDomains++;
+            }
+        }
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity: CUDA domains %d (%d device(s)), numCudaDomains, cudatopo->numDevices);
+        gpuDomains += numCudaDomains;
+    }
+#endif
+#ifdef LIKWID_WITH_ROCMON
+    if (doRocmDomains)
+    {
+        rocmtopo = get_rocmTopology();
+        for (int i = 0; i < rocmtopo->numDevices; i++)
+        {
+            RocmDevice* dev = &rocmtopo->devices[i];
+            if (dev->numaNode >= 0)
+            {
+                numRocmDomains++;
+            }
+        }
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity: ROCm domains %d (%d device(s)), numRocmDomains, rocmtopo->numDevices);
+        gpuDomains += numRocmDomains;
+    }
+#endif
+    numberOfDomains += gpuDomains;
+#endif
     DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity: All domains %d, numberOfDomains);
 
     domains = (AffinityDomain*) malloc(numberOfDomains * sizeof(AffinityDomain));
     if (!domains)
     {
         fprintf(stderr,"No more memory for %ld bytes for array of affinity domains\n",numberOfDomains * sizeof(AffinityDomain));
-        return;
+        return -ENOMEM;
     }
     memset(domains, 0, numberOfDomains * sizeof(AffinityDomain));
     int* helper = malloc(cputopo->numHWThreads * sizeof(int));
     if (!helper)
     {
-        return;
+        free(domains);
+        return -ENOMEM;
     }
 
     /* Node domain */
-    int err = affinity_addNodeDomain(&domains[domid], helper);
+    err = affinity_addNodeDomain(&domains[domid], helper);
     if (!err)
     {
         domid++;
@@ -638,6 +993,38 @@ affinity_init()
             finalNumberOfDomains++;
         }
     }
+#ifdef LIKWID_WITH_NVMON
+    if (doCudaDomains)
+    {
+        for (int i = 0; i < cudatopo->numDevices; i++)
+        {
+            err = affinity_addCudaDomain(i, &domains[domid], 0, helper);
+            if (!err)
+            {
+                domid++;
+                finalNumberOfDomains++;
+            }
+        }
+    }
+#endif
+#ifdef LIKWID_WITH_ROCMON
+    int gpuOffset = 0;
+#ifdef LIKWID_WITH_NVMON
+    gpuOffset = numCudaDomains;
+#endif
+    if (doRocmDomains)
+    {
+        for (int i = 0; i < rocmtopo->numDevices; i++)
+        {
+            err = affinity_addRocmDomain(i, &domains[domid], gpuOffset, helper);
+            if (!err)
+            {
+                domid++;
+                finalNumberOfDomains++;
+            }
+        }
+    }
+#endif
     free(helper);
 
     affinity_numberOfDomains = numberOfDomains;
@@ -648,15 +1035,24 @@ affinity_init()
     affinityDomains.numberOfCacheDomains = numberOfCacheDomains;
     affinityDomains.numberOfCoresPerCache = numberOfCoresPerCache;
     affinityDomains.numberOfProcessorsPerCache = numberOfProcessorsPerCache;
+    affinityDomains.numberOfCudaDomains = 0;
+#ifdef LIKWID_WITH_NVMON
+    affinityDomains.numberOfCudaDomains = numCudaDomains;
+#endif
+    affinityDomains.numberOfRocmDomains = 0;
+#ifdef LIKWID_WITH_ROCMON
+    affinityDomains.numberOfRocmDomains = numRocmDomains;
+#endif
     affinityDomains.domains = domains;
 
     create_lookups();
 
     affinity_initialized = 1;
+    return 0;
 }
 
 void
-affinity_finalize()
+affinity_finalize(void)
 {
     if (affinity_initialized == 0)
     {
@@ -668,8 +1064,10 @@ affinity_finalize()
     }
     for ( int i=0; i < affinityDomains.numberOfAffinityDomains; i++ )
     {
-        if (affinityDomains.domains[i].tag)
-            bdestroy(affinityDomains.domains[i].tag);
+        if (affinityDomains.domains[i].tag != NULL)
+        {
+            free(affinityDomains.domains[i].tag);
+        }
         if (affinityDomains.domains[i].processorList != NULL)
         {
             free(affinityDomains.domains[i].processorList);
@@ -756,7 +1154,7 @@ affinity_finalize()
 }
 
 int
-affinity_processGetProcessorId()
+affinity_processGetProcessorId(void)
 {
     int ret;
     cpu_set_t cpu_set;
@@ -772,7 +1170,7 @@ affinity_processGetProcessorId()
 }
 
 int
-affinity_threadGetProcessorId()
+affinity_threadGetProcessorId(void)
 {
     cpu_set_t  cpu_set;
     CPU_ZERO(&cpu_set);
@@ -825,12 +1223,12 @@ affinity_pinProcesses(int cpu_count, const int* processorIds)
 }
 
 const AffinityDomain*
-affinity_getDomain(bstring domain)
+affinity_getDomain(char* domain)
 {
 
     for ( int i=0; i < affinity_numberOfDomains; i++ )
     {
-        if ( biseq(domain, domains[i].tag) )
+        if ( strncmp(domains[i].tag, domain, strlen(domains[i].tag)) == 0 )
         {
             return domains+i;
         }
@@ -840,12 +1238,12 @@ affinity_getDomain(bstring domain)
 }
 
 void
-affinity_printDomains()
+affinity_printDomains(void)
 {
     for ( int i=0; i < affinity_numberOfDomains; i++ )
     {
         printf("Domain %d:\n",i);
-        printf("\tTag %s:",bdata(domains[i].tag));
+        printf("\tTag %s:", domains[i].tag);
 
         for ( uint32_t j=0; j < domains[i].numberOfProcessors; j++ )
         {
