@@ -7,13 +7,13 @@
  *      Description:  An application to read out performance counter registers
  *                    on x86 processors
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -925,6 +925,8 @@ end
 if print_info or verbose > 0 then
     print_stdout(string.format("CPU family:\t%u", cpuinfo["family"]))
     print_stdout(string.format("CPU model:\t%u", cpuinfo["model"]))
+    print_stdout(string.format("CPU vendor:\t%u", cpuinfo["vendor"]))
+    print_stdout(string.format("CPU part:\t%u", cpuinfo["part"]))
     print_stdout(string.format("CPU short:\t%s", cpuinfo["short_name"]))
     print_stdout(string.format("CPU stepping:\t%u", cpuinfo["stepping"]))
     print_stdout(string.format("CPU features:\t%s", cpuinfo["features"]))
@@ -936,7 +938,9 @@ if print_info or verbose > 0 then
         print_stdout(string.format("PERFMON number of counters:\t\t%u", cpuinfo["perf_num_ctr"]))
         print_stdout(string.format("PERFMON width of counters:\t\t%u", cpuinfo["perf_width_ctr"]))
         print_stdout(string.format("PERFMON number of fixed counters:\t%u", cpuinfo["perf_num_fixed_ctr"]))
+        print_stdout(string.format("PERFMON supports desktop memory controllers:\t%u", cpuinfo["supportClientmem"]))
     end
+    print_stdout(string.format("PERFMON supports Uncore:\t%u", cpuinfo["supportUncore"]))
     ---------------------------
     if nvSupported and cudatopo then
         print_stdout(likwid.hline)
@@ -1088,10 +1092,13 @@ if use_marker == true then
     likwid.setenv("LIKWID_FORCE", "-1")
     likwid.setenv("KMP_INIT_AT_FORK", "FALSE")
     if nvSupported and #gpulist_cuda > 0 and #cuda_event_string_list > 0 then
-        likwid.setenv("LIKWID_GPUS", table.concat(gpulist_cuda, ","))
+        likwid.setenv("LIKWID_NVMON_GPUS", table.concat(gpulist_cuda, ","))
         str = table.concat(cuda_event_string_list, "|")
-        likwid.setenv("LIKWID_GEVENTS", str)
-        likwid.setenv("LIKWID_GPUFILEPATH", nvMarkerFile)
+        likwid.setenv("LIKWID_NVMON_EVENTS", str)
+        likwid.setenv("LIKWID_NVMON_FILEPATH", nvMarkerFile)
+        if verbose > 0 then
+            likwid.setenv("LIKWID_NVMON_VERBOSITY", tostring(verbose))
+        end
     end
     if rocmSupported and #gpulist_rocm > 0 and #rocm_event_string_list > 0 then
         likwid.setenv("LIKWID_ROCMON_GPUS", table.concat(gpulist_rocm, ","))
@@ -1430,7 +1437,18 @@ if use_wrapper or use_timeline then
             end
             break
         end
-        local remain = likwid.sleep(math.floor(duration - (twork * 1E6)))
+	exitvalue, exited = likwid.checkProgram(pid)
+        if exited then
+            io.stdout:flush()
+            if #execList > 0 then
+                break
+            end
+        end
+        local sleeptime = duration - (twork * 1E6)
+	local remain = 0
+        if sleeptime > 0 then
+           remain = likwid.sleep(math.floor(duration - (twork * 1E6)))
+        end
 
         exitvalue, exited = likwid.checkProgram(pid)
         if exited then
@@ -1541,14 +1559,14 @@ if use_marker == true then
     ---------------------------
     if nvSupported and #cuda_event_string_list > 0 then
         if likwid.access(nvMarkerFile, "e") >= 0 then
-            results, metrics = likwid.getNvMarkerResults(nvMarkerFile, markergpulist, nan2value)
+            results, metrics = likwid.getMarkerResultsCuda(nvMarkerFile, gpulist_cuda, nan2value)
             if not results then
-                print_stderr("Failure reading GPU Marker API result file.")
+                print_stderr("Failure reading Nv Marker API result file.")
             elseif #results == 0 then
-                print_stderr("No regions could be found in GPU Marker API result file.")
+                print_stderr("No regions could be found in Nv Marker API result file.")
             else
                 for r = 1, #results do
-                    likwid.printGpuOutput(results[r], metrics[r], gpulist_cuda, r, print_stats)
+                    likwid.printOutputCuda(results[r], metrics[r], gpulist_cuda, r, print_stats)
                 end
             end
             likwid.destroyNvMarkerFile()
@@ -1561,7 +1579,7 @@ if use_marker == true then
     ---------------------------
     if rocmSupported and #rocm_event_string_list > 0 then
         if likwid.access(rocmMarkerFile, "e") >= 0 then
-            results, metrics = likwid.getMarkerResultsRocm(rocmMarkerFile, markerrocmgpulist, nan2value)
+            results, metrics = likwid.getMarkerResultsRocm(rocmMarkerFile, gpulist_rocm, nan2value)
             if not results then
                 print_stderr("Failure reading ROCM Marker API result file.")
             elseif #results == 0 then
