@@ -8,13 +8,13 @@
  *                    on x86, ARM and POWER processors. Also the Nvidia GPU topology
  *                    is printed.
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -144,6 +144,7 @@ table.insert(output_csv, "STRUCT,Hardware Thread Topology,3")
 table.insert(output_csv, "Hardware Thread Topology")
 table.insert(output_csv, likwid.sline)
 table.insert(output_csv, string.format("Sockets:\t\t%u",cputopo["numSockets"]))
+table.insert(output_csv, string.format("CPU dies:\t\t%u",cputopo["numDies"]))
 table.insert(output_csv, string.format("Cores per socket:\t%u",cputopo["numCoresPerSocket"]))
 table.insert(output_csv, string.format("Threads per core:\t%u",cputopo["numThreadsPerCore"]))
 table.insert(output_csv, likwid.hline)
@@ -329,6 +330,7 @@ if likwid.nvSupported() then
                 table.insert(output_csv, string.format("PCI bus:\t\t0x%x", gpu["pciBus"]))
                 table.insert(output_csv, string.format("PCI domain:\t\t0x%x", gpu["pciDom"]))
                 table.insert(output_csv, string.format("PCI device:\t\t0x%x", gpu["pciDev"]))
+                table.insert(output_csv, string.format("PCI function:\t\t0x%x", gpu["pciFunc"]))
             end
             table.insert(output_csv, likwid.hline)
         end
