@@ -9,13 +9,13 @@
  *                  Allows to turn on and off the Hardware prefetcher
  *                  available.
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -264,7 +264,7 @@ cpuFeatureNamesFixed[CPUFEATURES_MAX] = {};
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
 void
-cpuFeatures_init()
+cpuFeatures_init(void)
 {
     if (features_initialized)
     {
@@ -385,6 +385,8 @@ cpuFeatures_enable(int cpu, CpuFeature type, int print)
             (cpuid_info.model == ICELAKE1) ||
             (cpuid_info.model == ICELAKE2) ||
             (cpuid_info.model == SAPPHIRERAPIDS) ||
+            (cpuid_info.model == GRANITERAPIDS) ||
+            (cpuid_info.model == SIERRAFORREST) ||
             (cpuid_info.model == ROCKETLAKE) ||
             (cpuid_info.model == ICELAKEX1) ||
             (cpuid_info.model == ICELAKEX2) ||
@@ -550,6 +552,8 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
             (cpuid_info.model == ICELAKE1) ||
             (cpuid_info.model == ICELAKE2) ||
             (cpuid_info.model == SAPPHIRERAPIDS) ||
+            (cpuid_info.model == GRANITERAPIDS) ||
+            (cpuid_info.model == SIERRAFORREST) ||
             (cpuid_info.model == ROCKETLAKE) ||
             (cpuid_info.model == ICELAKEX1) ||
             (cpuid_info.model == ICELAKEX2) ||
