@@ -6,13 +6,13 @@
  *      Description:  Header File bitUtil Module.
  *                    Helper routines for dealing with bit manipulations
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -33,10 +33,12 @@
 
 #include <types.h>
 
-extern uint64_t field64(uint64_t value, int start, int length);
-extern uint32_t field32(uint32_t value, int start, int length);
-extern uint32_t extractBitField(uint32_t inField, uint32_t width, uint32_t offset);
-extern uint32_t getBitFieldWidth(uint32_t number);
+uint64_t field64(uint64_t bitfield, int start, int length);
+uint32_t field32(uint32_t bitfield, int start, int length);
+void field64set(uint64_t* bitfield, int start, int length, uint64_t value);
+void field32set(uint32_t* bitfield, int start, int length, uint32_t value);
+uint32_t extractBitField(uint32_t inField, uint32_t width, uint32_t offset);
+uint32_t getBitFieldWidth(uint32_t number);
 
 #define setBit(reg,bit)  (reg) |= (1ULL<<(bit))
 #define clearBit(reg,bit) (reg) &= ~(1ULL<<(bit))
