@@ -5,14 +5,14 @@
  *
  *      Description:  Types file for pci module.
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -53,6 +53,10 @@ typedef enum {
     PCI_R3QPI_DEVICE_LINK_1,
     PCI_R3QPI_DEVICE_LINK_2,
     PCI_R3QPI_DEVICE_LINK_3,
+    PCI_R3QPI_DEVICE_LINK_4,
+    PCI_R3QPI_DEVICE_LINK_5,
+    PCI_R3QPI_DEVICE_LINK_6,
+    PCI_R3QPI_DEVICE_LINK_7,
     PCI_R2PCIE_DEVICE0,
     PCI_R2PCIE_DEVICE1,
     PCI_R2PCIE_DEVICE2,
@@ -69,6 +73,30 @@ typedef enum {
     PCI_R2PCIE_DEVICE13,
     PCI_R2PCIE_DEVICE14,
     PCI_R2PCIE_DEVICE15,
+    PCI_R2PCIE_DEVICE16,
+    PCI_R2PCIE_DEVICE17,
+    PCI_R2PCIE_DEVICE18,
+    PCI_R2PCIE_DEVICE19,
+    PCI_R2PCIE_DEVICE20,
+    PCI_R2PCIE_DEVICE21,
+    PCI_R2PCIE_DEVICE22,
+    PCI_R2PCIE_DEVICE23,
+    PCI_R2PCIE_DEVICE24,
+    PCI_R2PCIE_DEVICE25,
+    PCI_R2PCIE_DEVICE26,
+    PCI_R2PCIE_DEVICE27,
+    PCI_R2PCIE_DEVICE28,
+    PCI_R2PCIE_DEVICE29,
+    PCI_R2PCIE_DEVICE30,
+    PCI_R2PCIE_DEVICE31,
+    PCI_R2PCIE_DEVICE32,
+    PCI_R2PCIE_DEVICE33,
+    PCI_R2PCIE_DEVICE34,
+    PCI_R2PCIE_DEVICE35,
+    PCI_R2PCIE_DEVICE36,
+    PCI_R2PCIE_DEVICE37,
+    PCI_R2PCIE_DEVICE38,
+    PCI_R2PCIE_DEVICE39,
     PCI_IMC_DEVICE_0_CH_0,
     PCI_IMC_DEVICE_0_CH_1,
     PCI_IMC_DEVICE_0_CH_2,
@@ -109,10 +137,42 @@ typedef enum {
     PCI_HA_DEVICE_29,
     PCI_HA_DEVICE_30,
     PCI_HA_DEVICE_31,
+    PCI_HA_DEVICE_32,
+    PCI_HA_DEVICE_33,
+    PCI_HA_DEVICE_34,
+    PCI_HA_DEVICE_35,
+    PCI_HA_DEVICE_36,
+    PCI_HA_DEVICE_37,
+    PCI_HA_DEVICE_38,
+    PCI_HA_DEVICE_39,
+    PCI_HA_DEVICE_40,
+    PCI_HA_DEVICE_41,
+    PCI_HA_DEVICE_42,
+    PCI_HA_DEVICE_43,
+    PCI_HA_DEVICE_44,
+    PCI_HA_DEVICE_45,
+    PCI_HA_DEVICE_46,
+    PCI_HA_DEVICE_47,
+    PCI_HA_DEVICE_48,
+    PCI_HA_DEVICE_49,
+    PCI_HA_DEVICE_50,
+    PCI_HA_DEVICE_51,
+    PCI_HA_DEVICE_52,
+    PCI_HA_DEVICE_53,
+    PCI_HA_DEVICE_54,
+    PCI_HA_DEVICE_55,
+    PCI_HA_DEVICE_56,
+    PCI_HA_DEVICE_57,
+    PCI_HA_DEVICE_58,
+    PCI_HA_DEVICE_59,
     PCI_QPI_DEVICE_PORT_0,
     PCI_QPI_DEVICE_PORT_1,
     PCI_QPI_DEVICE_PORT_2,
     PCI_QPI_DEVICE_PORT_3,
+    PCI_QPI_DEVICE_PORT_4,
+    PCI_QPI_DEVICE_PORT_5,
+    PCI_QPI_DEVICE_PORT_6,
+    PCI_QPI_DEVICE_PORT_7,
     PCI_QPI_MASK_DEVICE_PORT_0,
     PCI_QPI_MASK_DEVICE_PORT_1,
     PCI_QPI_MASK_DEVICE_PORT_2,
@@ -121,7 +181,6 @@ typedef enum {
     PCI_QPI_MISC_DEVICE_PORT_1,
     PCI_QPI_MISC_DEVICE_PORT_2,
     PCI_QPI_MISC_DEVICE_PORT_3,
-    PCI_IRP_DEVICE,
     PCI_EDC0_UCLK_DEVICE,
     PCI_EDC0_DCLK_DEVICE,
     PCI_EDC1_UCLK_DEVICE,
@@ -266,14 +325,89 @@ typedef enum {
     MSR_CBOX_DEVICE_C57,
     MSR_CBOX_DEVICE_C58,
     MSR_CBOX_DEVICE_C59,
-    MSR_PBOX_DEVICE,
+    MSR_CBOX_DEVICE_C60,
+    MSR_CBOX_DEVICE_C61,
+    MSR_CBOX_DEVICE_C62,
+    MSR_CBOX_DEVICE_C63,
+    MSR_CBOX_DEVICE_C64,
+    MSR_CBOX_DEVICE_C65,
+    MSR_CBOX_DEVICE_C66,
+    MSR_CBOX_DEVICE_C67,
+    MSR_CBOX_DEVICE_C68,
+    MSR_CBOX_DEVICE_C69,
+    MSR_CBOX_DEVICE_C70,
+    MSR_CBOX_DEVICE_C71,
+    MSR_CBOX_DEVICE_C72,
+    MSR_CBOX_DEVICE_C73,
+    MSR_CBOX_DEVICE_C74,
+    MSR_CBOX_DEVICE_C75,
+    MSR_CBOX_DEVICE_C76,
+    MSR_CBOX_DEVICE_C77,
+    MSR_CBOX_DEVICE_C78,
+    MSR_CBOX_DEVICE_C79,
+    MSR_CBOX_DEVICE_C80,
+    MSR_CBOX_DEVICE_C81,
+    MSR_CBOX_DEVICE_C82,
+    MSR_CBOX_DEVICE_C83,
+    MSR_CBOX_DEVICE_C84,
+    MSR_CBOX_DEVICE_C85,
+    MSR_CBOX_DEVICE_C86,
+    MSR_CBOX_DEVICE_C87,
+    MSR_CBOX_DEVICE_C88,
+    MSR_CBOX_DEVICE_C89,
+    MSR_CBOX_DEVICE_C90,
+    MSR_CBOX_DEVICE_C91,
+    MSR_CBOX_DEVICE_C92,
+    MSR_CBOX_DEVICE_C93,
+    MSR_CBOX_DEVICE_C94,
+    MSR_CBOX_DEVICE_C95,
+    MSR_CBOX_DEVICE_C96,
+    MSR_CBOX_DEVICE_C97,
+    MSR_CBOX_DEVICE_C98,
+    MSR_CBOX_DEVICE_C99,
+    MSR_CBOX_DEVICE_C100,
+    MSR_CBOX_DEVICE_C101,
+    MSR_CBOX_DEVICE_C102,
+    MSR_CBOX_DEVICE_C103,
+    MSR_CBOX_DEVICE_C104,
+    MSR_CBOX_DEVICE_C105,
+    MSR_CBOX_DEVICE_C106,
+    MSR_CBOX_DEVICE_C107,
+    MSR_CBOX_DEVICE_C108,
+    MSR_CBOX_DEVICE_C109,
+    MSR_CBOX_DEVICE_C110,
+    MSR_CBOX_DEVICE_C111,
+    MSR_CBOX_DEVICE_C112,
+    MSR_CBOX_DEVICE_C113,
+    MSR_CBOX_DEVICE_C114,
+    MSR_CBOX_DEVICE_C115,
+    MSR_CBOX_DEVICE_C116,
+    MSR_CBOX_DEVICE_C117,
+    MSR_CBOX_DEVICE_C118,
+    MSR_CBOX_DEVICE_C119,
+    MSR_CBOX_DEVICE_C120,
+    MSR_CBOX_DEVICE_C121,
+    MSR_CBOX_DEVICE_C122,
+    MSR_CBOX_DEVICE_C123,
+    MSR_CBOX_DEVICE_C124,
+    MSR_CBOX_DEVICE_C125,
+    MSR_CBOX_DEVICE_C126,
+    MSR_CBOX_DEVICE_C127,
+    MSR_CBOX_DEVICE_C128,
+    MSR_CBOX_DEVICE_C129,
+    MSR_PBOX_DEVICE_0,
     MSR_PBOX_DEVICE_1,
     MSR_PBOX_DEVICE_2,
     MSR_PBOX_DEVICE_3,
     MSR_PBOX_DEVICE_4,
     MSR_PBOX_DEVICE_5,
-    MSR_PCU_DEVICE,
-    MSR_IIO_DEVICE,
+    MSR_PBOX_DEVICE_6,
+    MSR_PBOX_DEVICE_7,
+    MSR_PBOX_DEVICE_8,
+    MSR_PBOX_DEVICE_9,
+    MSR_PBOX_DEVICE_10,
+    MSR_PBOX_DEVICE_11,
+    MSR_PBOX_DEVICE_12,
     MSR_MDF_DEVICE_0,
     MSR_MDF_DEVICE_1,
     MSR_MDF_DEVICE_2,
@@ -324,6 +458,46 @@ typedef enum {
     MSR_MDF_DEVICE_47,
     MSR_MDF_DEVICE_48,
     MSR_MDF_DEVICE_49,
+    MSR_MDF_DEVICE_50,
+    MSR_MDF_DEVICE_51,
+    MSR_MDF_DEVICE_52,
+    MSR_MDF_DEVICE_53,
+    MSR_MDF_DEVICE_54,
+    MSR_MDF_DEVICE_55,
+    MSR_MDF_DEVICE_56,
+    MSR_MDF_DEVICE_57,
+    MSR_MDF_DEVICE_58,
+    MSR_MDF_DEVICE_59,
+    MSR_MDF_DEVICE_60,
+    MSR_MDF_DEVICE_61,
+    MSR_MDF_DEVICE_62,
+    MSR_MDF_DEVICE_63,
+    MSR_MDF_DEVICE_64,
+    MSR_MDF_DEVICE_65,
+    MSR_MDF_DEVICE_66,
+    MSR_MDF_DEVICE_67,
+    MSR_MDF_DEVICE_68,
+    MSR_MDF_DEVICE_69,
+    MSR_MDF_DEVICE_70,
+    MSR_MDF_DEVICE_71,
+    MSR_MDF_DEVICE_72,
+    MSR_MDF_DEVICE_73,
+    MSR_MDF_DEVICE_74,
+    MSR_MDF_DEVICE_75,
+    MSR_MDF_DEVICE_76,
+    MSR_MDF_DEVICE_77,
+    MSR_MDF_DEVICE_78,
+    MSR_MDF_DEVICE_79,
+    MSR_MDF_DEVICE_80,
+    MSR_MDF_DEVICE_81,
+    MSR_MDF_DEVICE_82,
+    MSR_MDF_DEVICE_83,
+    MSR_MDF_DEVICE_84,
+    MSR_MDF_DEVICE_85,
+    MSR_MDF_DEVICE_86,
+    MSR_MDF_DEVICE_87,
+    MSR_MDF_DEVICE_88,
+    MSR_MDF_DEVICE_89,
     MSR_UBOX_DEVICE,
     MSR_IIO_DEVICE_0,
     MSR_IIO_DEVICE_1,
@@ -365,11 +539,22 @@ typedef enum {
     MSR_M2IOSF_DEVICE_5,
     MSR_M2IOSF_DEVICE_6,
     MSR_M2IOSF_DEVICE_7,
+    MSR_PCU_DEVICE_0,
+    MSR_PCU_DEVICE_1,
+    MSR_PCU_DEVICE_2,
+    MSR_PCU_DEVICE_3,
+    MSR_PCU_DEVICE_4,
+    MSR_PCU_DEVICE_5,
+    MSR_PCU_DEVICE_6,
+    MSR_PCU_DEVICE_7,
     MAX_NUM_PCI_DEVICES
 } PciDeviceIndex;
 
-#define MSR_PBOX_DEVICE_0 MSR_PBOX_DEVICE
+#define MSR_PBOX_DEVICE MSR_PBOX_DEVICE_0
 #define PCI_R2PCIE_DEVICE PCI_R2PCIE_DEVICE0
+#define MSR_IIO_DEVICE MSR_IIO_DEVICE_0
+#define PCI_IRP_DEVICE MSR_IRP_DEVICE_0
+#define MSR_PCU_DEVICE MSR_PCU_DEVICE_0
 
 typedef struct {
     PciDeviceType type;
@@ -504,6 +689,80 @@ static char* pci_device_names[MAX_NUM_PCI_DEVICES] = {
     [MSR_CBOX_DEVICE_C53] = MYSTRING(MSR_CBOX_DEVICE_C53),
     [MSR_CBOX_DEVICE_C54] = MYSTRING(MSR_CBOX_DEVICE_C54),
     [MSR_CBOX_DEVICE_C55] = MYSTRING(MSR_CBOX_DEVICE_C55),
+    [MSR_CBOX_DEVICE_C56] = MYSTRING(MSR_CBOX_DEVICE_C56),
+    [MSR_CBOX_DEVICE_C57] = MYSTRING(MSR_CBOX_DEVICE_C57),
+    [MSR_CBOX_DEVICE_C58] = MYSTRING(MSR_CBOX_DEVICE_C58),
+    [MSR_CBOX_DEVICE_C59] = MYSTRING(MSR_CBOX_DEVICE_C59),
+    [MSR_CBOX_DEVICE_C60] = MYSTRING(MSR_CBOX_DEVICE_C60),
+    [MSR_CBOX_DEVICE_C61] = MYSTRING(MSR_CBOX_DEVICE_C61),
+    [MSR_CBOX_DEVICE_C62] = MYSTRING(MSR_CBOX_DEVICE_C62),
+    [MSR_CBOX_DEVICE_C63] = MYSTRING(MSR_CBOX_DEVICE_C63),
+    [MSR_CBOX_DEVICE_C64] = MYSTRING(MSR_CBOX_DEVICE_C64),
+    [MSR_CBOX_DEVICE_C65] = MYSTRING(MSR_CBOX_DEVICE_C65),
+    [MSR_CBOX_DEVICE_C66] = MYSTRING(MSR_CBOX_DEVICE_C66),
+    [MSR_CBOX_DEVICE_C67] = MYSTRING(MSR_CBOX_DEVICE_C67),
+    [MSR_CBOX_DEVICE_C68] = MYSTRING(MSR_CBOX_DEVICE_C68),
+    [MSR_CBOX_DEVICE_C69] = MYSTRING(MSR_CBOX_DEVICE_C69),
+    [MSR_CBOX_DEVICE_C70] = MYSTRING(MSR_CBOX_DEVICE_C70),
+    [MSR_CBOX_DEVICE_C71] = MYSTRING(MSR_CBOX_DEVICE_C71),
+    [MSR_CBOX_DEVICE_C72] = MYSTRING(MSR_CBOX_DEVICE_C72),
+    [MSR_CBOX_DEVICE_C73] = MYSTRING(MSR_CBOX_DEVICE_C73),
+    [MSR_CBOX_DEVICE_C74] = MYSTRING(MSR_CBOX_DEVICE_C74),
+    [MSR_CBOX_DEVICE_C75] = MYSTRING(MSR_CBOX_DEVICE_C75),
+    [MSR_CBOX_DEVICE_C76] = MYSTRING(MSR_CBOX_DEVICE_C76),
+    [MSR_CBOX_DEVICE_C77] = MYSTRING(MSR_CBOX_DEVICE_C77),
+    [MSR_CBOX_DEVICE_C78] = MYSTRING(MSR_CBOX_DEVICE_C78),
+    [MSR_CBOX_DEVICE_C79] = MYSTRING(MSR_CBOX_DEVICE_C79),
+    [MSR_CBOX_DEVICE_C80] = MYSTRING(MSR_CBOX_DEVICE_C80),
+    [MSR_CBOX_DEVICE_C81] = MYSTRING(MSR_CBOX_DEVICE_C81),
+    [MSR_CBOX_DEVICE_C82] = MYSTRING(MSR_CBOX_DEVICE_C82),
+    [MSR_CBOX_DEVICE_C83] = MYSTRING(MSR_CBOX_DEVICE_C83),
+    [MSR_CBOX_DEVICE_C84] = MYSTRING(MSR_CBOX_DEVICE_C84),
+    [MSR_CBOX_DEVICE_C85] = MYSTRING(MSR_CBOX_DEVICE_C85),
+    [MSR_CBOX_DEVICE_C86] = MYSTRING(MSR_CBOX_DEVICE_C86),
+    [MSR_CBOX_DEVICE_C87] = MYSTRING(MSR_CBOX_DEVICE_C87),
+    [MSR_CBOX_DEVICE_C88] = MYSTRING(MSR_CBOX_DEVICE_C88),
+    [MSR_CBOX_DEVICE_C89] = MYSTRING(MSR_CBOX_DEVICE_C89),
+    [MSR_CBOX_DEVICE_C90] = MYSTRING(MSR_CBOX_DEVICE_C90),
+    [MSR_CBOX_DEVICE_C91] = MYSTRING(MSR_CBOX_DEVICE_C91),
+    [MSR_CBOX_DEVICE_C92] = MYSTRING(MSR_CBOX_DEVICE_C92),
+    [MSR_CBOX_DEVICE_C93] = MYSTRING(MSR_CBOX_DEVICE_C93),
+    [MSR_CBOX_DEVICE_C94] = MYSTRING(MSR_CBOX_DEVICE_C94),
+    [MSR_CBOX_DEVICE_C95] = MYSTRING(MSR_CBOX_DEVICE_C95),
+    [MSR_CBOX_DEVICE_C96] = MYSTRING(MSR_CBOX_DEVICE_C96),
+    [MSR_CBOX_DEVICE_C97] = MYSTRING(MSR_CBOX_DEVICE_C97),
+    [MSR_CBOX_DEVICE_C98] = MYSTRING(MSR_CBOX_DEVICE_C98),
+    [MSR_CBOX_DEVICE_C99] = MYSTRING(MSR_CBOX_DEVICE_C99),
+    [MSR_CBOX_DEVICE_C100] = MYSTRING(MSR_CBOX_DEVICE_C100),
+    [MSR_CBOX_DEVICE_C101] = MYSTRING(MSR_CBOX_DEVICE_C101),
+    [MSR_CBOX_DEVICE_C102] = MYSTRING(MSR_CBOX_DEVICE_C102),
+    [MSR_CBOX_DEVICE_C103] = MYSTRING(MSR_CBOX_DEVICE_C103),
+    [MSR_CBOX_DEVICE_C104] = MYSTRING(MSR_CBOX_DEVICE_C104),
+    [MSR_CBOX_DEVICE_C105] = MYSTRING(MSR_CBOX_DEVICE_C105),
+    [MSR_CBOX_DEVICE_C106] = MYSTRING(MSR_CBOX_DEVICE_C106),
+    [MSR_CBOX_DEVICE_C107] = MYSTRING(MSR_CBOX_DEVICE_C107),
+    [MSR_CBOX_DEVICE_C108] = MYSTRING(MSR_CBOX_DEVICE_C108),
+    [MSR_CBOX_DEVICE_C109] = MYSTRING(MSR_CBOX_DEVICE_C109),
+    [MSR_CBOX_DEVICE_C110] = MYSTRING(MSR_CBOX_DEVICE_C110),
+    [MSR_CBOX_DEVICE_C111] = MYSTRING(MSR_CBOX_DEVICE_C111),
+    [MSR_CBOX_DEVICE_C112] = MYSTRING(MSR_CBOX_DEVICE_C112),
+    [MSR_CBOX_DEVICE_C113] = MYSTRING(MSR_CBOX_DEVICE_C113),
+    [MSR_CBOX_DEVICE_C114] = MYSTRING(MSR_CBOX_DEVICE_C114),
+    [MSR_CBOX_DEVICE_C115] = MYSTRING(MSR_CBOX_DEVICE_C115),
+    [MSR_CBOX_DEVICE_C116] = MYSTRING(MSR_CBOX_DEVICE_C116),
+    [MSR_CBOX_DEVICE_C117] = MYSTRING(MSR_CBOX_DEVICE_C117),
+    [MSR_CBOX_DEVICE_C118] = MYSTRING(MSR_CBOX_DEVICE_C118),
+    [MSR_CBOX_DEVICE_C119] = MYSTRING(MSR_CBOX_DEVICE_C119),
+    [MSR_CBOX_DEVICE_C120] = MYSTRING(MSR_CBOX_DEVICE_C120),
+    [MSR_CBOX_DEVICE_C121] = MYSTRING(MSR_CBOX_DEVICE_C121),
+    [MSR_CBOX_DEVICE_C122] = MYSTRING(MSR_CBOX_DEVICE_C122),
+    [MSR_CBOX_DEVICE_C123] = MYSTRING(MSR_CBOX_DEVICE_C123),
+    [MSR_CBOX_DEVICE_C124] = MYSTRING(MSR_CBOX_DEVICE_C124),
+    [MSR_CBOX_DEVICE_C125] = MYSTRING(MSR_CBOX_DEVICE_C125),
+    [MSR_CBOX_DEVICE_C126] = MYSTRING(MSR_CBOX_DEVICE_C126),
+    [MSR_CBOX_DEVICE_C127] = MYSTRING(MSR_CBOX_DEVICE_C127),
+    [MSR_CBOX_DEVICE_C128] = MYSTRING(MSR_CBOX_DEVICE_C128),
+    [MSR_CBOX_DEVICE_C129] = MYSTRING(MSR_CBOX_DEVICE_C129),
     [MSR_UBOX_DEVICE] = MYSTRING(MSR_UBOX_DEVICE),
     [MSR_MDF_DEVICE_0] = MYSTRING(MSR_MDF_DEVICE_0),
     [MSR_MDF_DEVICE_1] = MYSTRING(MSR_MDF_DEVICE_1),
@@ -555,6 +814,46 @@ static char* pci_device_names[MAX_NUM_PCI_DEVICES] = {
     [MSR_MDF_DEVICE_47] = MYSTRING(MSR_MDF_DEVICE_47),
     [MSR_MDF_DEVICE_48] = MYSTRING(MSR_MDF_DEVICE_48),
     [MSR_MDF_DEVICE_49] = MYSTRING(MSR_MDF_DEVICE_49),
+    [MSR_MDF_DEVICE_50] = MYSTRING(MSR_MDF_DEVICE_50),
+    [MSR_MDF_DEVICE_51] = MYSTRING(MSR_MDF_DEVICE_51),
+    [MSR_MDF_DEVICE_52] = MYSTRING(MSR_MDF_DEVICE_52),
+    [MSR_MDF_DEVICE_53] = MYSTRING(MSR_MDF_DEVICE_53),
+    [MSR_MDF_DEVICE_54] = MYSTRING(MSR_MDF_DEVICE_54),
+    [MSR_MDF_DEVICE_55] = MYSTRING(MSR_MDF_DEVICE_55),
+    [MSR_MDF_DEVICE_56] = MYSTRING(MSR_MDF_DEVICE_56),
+    [MSR_MDF_DEVICE_57] = MYSTRING(MSR_MDF_DEVICE_57),
+    [MSR_MDF_DEVICE_58] = MYSTRING(MSR_MDF_DEVICE_58),
+    [MSR_MDF_DEVICE_59] = MYSTRING(MSR_MDF_DEVICE_59),
+    [MSR_MDF_DEVICE_60] = MYSTRING(MSR_MDF_DEVICE_60),
+    [MSR_MDF_DEVICE_61] = MYSTRING(MSR_MDF_DEVICE_61),
+    [MSR_MDF_DEVICE_62] = MYSTRING(MSR_MDF_DEVICE_62),
+    [MSR_MDF_DEVICE_63] = MYSTRING(MSR_MDF_DEVICE_63),
+    [MSR_MDF_DEVICE_64] = MYSTRING(MSR_MDF_DEVICE_64),
+    [MSR_MDF_DEVICE_65] = MYSTRING(MSR_MDF_DEVICE_65),
+    [MSR_MDF_DEVICE_66] = MYSTRING(MSR_MDF_DEVICE_66),
+    [MSR_MDF_DEVICE_67] = MYSTRING(MSR_MDF_DEVICE_67),
+    [MSR_MDF_DEVICE_68] = MYSTRING(MSR_MDF_DEVICE_68),
+    [MSR_MDF_DEVICE_69] = MYSTRING(MSR_MDF_DEVICE_69),
+    [MSR_MDF_DEVICE_70] = MYSTRING(MSR_MDF_DEVICE_70),
+    [MSR_MDF_DEVICE_71] = MYSTRING(MSR_MDF_DEVICE_71),
+    [MSR_MDF_DEVICE_72] = MYSTRING(MSR_MDF_DEVICE_72),
+    [MSR_MDF_DEVICE_73] = MYSTRING(MSR_MDF_DEVICE_73),
+    [MSR_MDF_DEVICE_74] = MYSTRING(MSR_MDF_DEVICE_74),
+    [MSR_MDF_DEVICE_75] = MYSTRING(MSR_MDF_DEVICE_75),
+    [MSR_MDF_DEVICE_76] = MYSTRING(MSR_MDF_DEVICE_76),
+    [MSR_MDF_DEVICE_77] = MYSTRING(MSR_MDF_DEVICE_77),
+    [MSR_MDF_DEVICE_78] = MYSTRING(MSR_MDF_DEVICE_78),
+    [MSR_MDF_DEVICE_79] = MYSTRING(MSR_MDF_DEVICE_79),
+    [MSR_MDF_DEVICE_80] = MYSTRING(MSR_MDF_DEVICE_80),
+    [MSR_MDF_DEVICE_81] = MYSTRING(MSR_MDF_DEVICE_81),
+    [MSR_MDF_DEVICE_82] = MYSTRING(MSR_MDF_DEVICE_82),
+    [MSR_MDF_DEVICE_83] = MYSTRING(MSR_MDF_DEVICE_83),
+    [MSR_MDF_DEVICE_84] = MYSTRING(MSR_MDF_DEVICE_84),
+    [MSR_MDF_DEVICE_85] = MYSTRING(MSR_MDF_DEVICE_85),
+    [MSR_MDF_DEVICE_86] = MYSTRING(MSR_MDF_DEVICE_86),
+    [MSR_MDF_DEVICE_87] = MYSTRING(MSR_MDF_DEVICE_87),
+    [MSR_MDF_DEVICE_88] = MYSTRING(MSR_MDF_DEVICE_88),
+    [MSR_MDF_DEVICE_89] = MYSTRING(MSR_MDF_DEVICE_89),
     [PCI_HA_DEVICE_0] = MYSTRING(PCI_HA_DEVICE_0),
     [PCI_HA_DEVICE_1] = MYSTRING(PCI_HA_DEVICE_1),
     [PCI_HA_DEVICE_2] = MYSTRING(PCI_HA_DEVICE_2),
@@ -591,11 +890,20 @@ static char* pci_device_names[MAX_NUM_PCI_DEVICES] = {
     [PCI_QPI_DEVICE_PORT_1] = MYSTRING(PCI_QPI_DEVICE_PORT_1),
     [PCI_QPI_DEVICE_PORT_2] = MYSTRING(PCI_QPI_DEVICE_PORT_2),
     [PCI_QPI_DEVICE_PORT_3] = MYSTRING(PCI_QPI_DEVICE_PORT_3),
+    [PCI_QPI_DEVICE_PORT_4] = MYSTRING(PCI_QPI_DEVICE_PORT_4),
+    [PCI_QPI_DEVICE_PORT_5] = MYSTRING(PCI_QPI_DEVICE_PORT_5),
     [PCI_R3QPI_DEVICE_LINK_0] = MYSTRING(PCI_R3QPI_DEVICE_LINK_0),
     [PCI_R3QPI_DEVICE_LINK_1] = MYSTRING(PCI_R3QPI_DEVICE_LINK_1),
     [PCI_R3QPI_DEVICE_LINK_2] = MYSTRING(PCI_R3QPI_DEVICE_LINK_2),
     [PCI_R3QPI_DEVICE_LINK_3] = MYSTRING(PCI_R3QPI_DEVICE_LINK_3),
-    [MSR_PCU_DEVICE] = MYSTRING(MSR_PCU_DEVICE),
+    [PCI_R3QPI_DEVICE_LINK_4] = MYSTRING(PCI_R3QPI_DEVICE_LINK_4),
+    [PCI_R3QPI_DEVICE_LINK_5] = MYSTRING(PCI_R3QPI_DEVICE_LINK_5),
+    [MSR_PCU_DEVICE_0] = MYSTRING(MSR_PCU_DEVICE_0),
+    [MSR_PCU_DEVICE_1] = MYSTRING(MSR_PCU_DEVICE_1),
+    [MSR_PCU_DEVICE_2] = MYSTRING(MSR_PCU_DEVICE_2),
+    [MSR_PCU_DEVICE_3] = MYSTRING(MSR_PCU_DEVICE_3),
+    [MSR_PCU_DEVICE_4] = MYSTRING(MSR_PCU_DEVICE_4),
+    [MSR_PCU_DEVICE_5] = MYSTRING(MSR_PCU_DEVICE_5),
     [MSR_IRP_DEVICE_0] = MYSTRING(MSR_IRP_DEVICE_0),
     [MSR_IRP_DEVICE_1] = MYSTRING(MSR_IRP_DEVICE_1),
     [MSR_IRP_DEVICE_2] = MYSTRING(MSR_IRP_DEVICE_2),
@@ -644,6 +952,31 @@ static char* pci_device_names[MAX_NUM_PCI_DEVICES] = {
     [PCI_R2PCIE_DEVICE13] = MYSTRING(PCI_R2PCIE_DEVICE13),
     [PCI_R2PCIE_DEVICE14] = MYSTRING(PCI_R2PCIE_DEVICE14),
     [PCI_R2PCIE_DEVICE15] = MYSTRING(PCI_R2PCIE_DEVICE15),
+    [PCI_R2PCIE_DEVICE16] = MYSTRING(PCI_R2PCIE_DEVICE16),
+    [PCI_R2PCIE_DEVICE17] = MYSTRING(PCI_R2PCIE_DEVICE17),
+    [PCI_R2PCIE_DEVICE18] = MYSTRING(PCI_R2PCIE_DEVICE18),
+    [PCI_R2PCIE_DEVICE19] = MYSTRING(PCI_R2PCIE_DEVICE19),
+    [PCI_R2PCIE_DEVICE20] = MYSTRING(PCI_R2PCIE_DEVICE20),
+    [PCI_R2PCIE_DEVICE21] = MYSTRING(PCI_R2PCIE_DEVICE21),
+    [PCI_R2PCIE_DEVICE22] = MYSTRING(PCI_R2PCIE_DEVICE22),
+    [PCI_R2PCIE_DEVICE23] = MYSTRING(PCI_R2PCIE_DEVICE23),
+    [PCI_R2PCIE_DEVICE24] = MYSTRING(PCI_R2PCIE_DEVICE24),
+    [PCI_R2PCIE_DEVICE25] = MYSTRING(PCI_R2PCIE_DEVICE25),
+    [PCI_R2PCIE_DEVICE26] = MYSTRING(PCI_R2PCIE_DEVICE26),
+    [PCI_R2PCIE_DEVICE27] = MYSTRING(PCI_R2PCIE_DEVICE27),
+    [PCI_R2PCIE_DEVICE28] = MYSTRING(PCI_R2PCIE_DEVICE28),
+    [PCI_R2PCIE_DEVICE29] = MYSTRING(PCI_R2PCIE_DEVICE29),
+    [PCI_R2PCIE_DEVICE30] = MYSTRING(PCI_R2PCIE_DEVICE30),
+    [PCI_R2PCIE_DEVICE31] = MYSTRING(PCI_R2PCIE_DEVICE31),
+    [PCI_R2PCIE_DEVICE32] = MYSTRING(PCI_R2PCIE_DEVICE32),
+    [PCI_R2PCIE_DEVICE33] = MYSTRING(PCI_R2PCIE_DEVICE33),
+    [PCI_R2PCIE_DEVICE34] = MYSTRING(PCI_R2PCIE_DEVICE34),
+    [PCI_R2PCIE_DEVICE35] = MYSTRING(PCI_R2PCIE_DEVICE35),
+    [PCI_R2PCIE_DEVICE36] = MYSTRING(PCI_R2PCIE_DEVICE36),
+    [PCI_R2PCIE_DEVICE37] = MYSTRING(PCI_R2PCIE_DEVICE37),
+    [PCI_R2PCIE_DEVICE38] = MYSTRING(PCI_R2PCIE_DEVICE38),
+    [PCI_R2PCIE_DEVICE39] = MYSTRING(PCI_R2PCIE_DEVICE39),
+
 
 };
 
