@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of perfmon module for perf_event kernel interface.
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -193,7 +193,7 @@ int perfmon_init_perfevent(int cpu_id)
         active_cpus += 1;
     }
     perf_event_num_cpus = cpuid_topology.numHWThreads;
-    if (cpuid_info.family == ZEN3_FAMILY && (cpuid_info.model == ZEN4_RYZEN || cpuid_info.model == ZEN4_EPYC))
+    if (cpuid_info.family == ZEN3_FAMILY && (cpuid_info.model == ZEN4_RYZEN || cpuid_info.model == ZEN4_RYZEN2 || cpuid_info.model == ZEN4_RYZEN_PRO || cpuid_info.model == ZEN4_EPYC ))
     {
         perfEventOptionNames[EVENT_OPTION_TID] = "threadmask";
         perfEventOptionNames[EVENT_OPTION_CID] = "coreid";
@@ -718,13 +718,32 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
     FILE* fp = NULL;
     int perf_type = 0;
     uint64_t eventConfig = 0x0;
+    bstring perf_folder = NULL;
+    struct bstrList* folders = NULL;
     if (perf_event_paranoid > 0 && getuid() != 0)
     {
         return EPERM;
     }
     attr->type = 0;
-    DEBUG_PRINT(DEBUGLEV_DEVELOP, Get information for uncore counters from folder %s, translate_types[type]);
-    perf_type = read_perf_event_type(translate_types[type]);
+    DEBUG_PRINT(DEBUGLEV_DEVELOP, Get information for uncore counters from folder(s): %s, translate_types[type]);
+    perf_folder = bfromcstr(translate_types[type]);
+    folders = bsplit(perf_folder, ' ');
+    bdestroy(perf_folder);
+    perf_folder = NULL;
+    for (int i = 0; i < folders->qty; i++)
+    {
+        if (!access(bdata(folders->entry[i]), R_OK|X_OK))
+        {
+            perf_folder = bstrcpy(folders->entry[i]);
+            break;
+        }
+    }
+    bstrListDestroy(folders);
+    if (!perf_folder)
+    {
+        return 1;
+    }
+    perf_type = read_perf_event_type(bdata(perf_folder));
     if (perf_type < 0)
     {
         if ((type == UBOX)||(type == UBOXFIX))
@@ -745,7 +764,7 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
 #else
     eventConfig = event->eventId;
 #endif
-    ret = parse_and_apply_event_config(translate_types[type], perfEventOptionNames[EVENT_OPTION_GENERIC_CONFIG], eventConfig, attr);
+    ret = parse_and_apply_event_config(bdata(perf_folder), perfEventOptionNames[EVENT_OPTION_GENERIC_CONFIG], eventConfig, attr);
 /*    num_formats = 0;*/
 /*    formats = NULL;*/
 /*    ret = parse_event_config(translate_types[type], perfEventOptionNames[EVENT_OPTION_GENERIC_CONFIG], &num_formats, &formats);*/
@@ -775,7 +794,7 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
     {
         num_formats = 0;
         formats = NULL;
-        ret = parse_event_config(translate_types[type], perfEventOptionNames[EVENT_OPTION_UNCORE_CONFIG], &num_formats, &formats);
+        ret = parse_event_config(bdata(perf_folder), perfEventOptionNames[EVENT_OPTION_UNCORE_CONFIG], &num_formats, &formats);
         if (ret == 0)
         {
             for (int i = 0; i < num_formats && umask != 0x0; i++)
@@ -802,7 +821,7 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
     {
         num_formats = 0;
         formats = NULL;
-        ret = parse_event_config(translate_types[type], perfEventOptionNames[EVENT_OPTION_GENERIC_UMASK], &num_formats, &formats);
+        ret = parse_event_config(bdata(perf_folder), perfEventOptionNames[EVENT_OPTION_GENERIC_UMASK], &num_formats, &formats);
         if (ret == 0)
         {
             uint64_t umask = event->umask;
@@ -865,7 +884,7 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
                 case EVENT_OPTION_SLICE:
                     num_formats = 0;
                     formats = NULL;
-                    ret = parse_event_config(translate_types[type], perfEventOptionNames[event->options[j].type], &num_formats, &formats);
+                    ret = parse_event_config(bdata(perf_folder), perfEventOptionNames[event->options[j].type], &num_formats, &formats);
                     if (ret == 0)
                     {
                         uint64_t optval = event->options[j].value;
@@ -901,7 +920,8 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
             }
         }
     }
-    if (type != POWER && cpuid_info.family == ZEN3_FAMILY && (cpuid_info.model == ZEN4_RYZEN || cpuid_info.model == ZEN4_EPYC))
+
+    if (type != POWER && cpuid_info.family == ZEN3_FAMILY && (cpuid_info.model == ZEN4_RYZEN || cpuid_info.model == ZEN4_RYZEN2 || cpuid_info.model == ZEN4_RYZEN_PRO || cpuid_info.model == ZEN4_EPYC))
     {
         int got_cid = 0;
         int got_slices = 0;
@@ -937,6 +957,7 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
             attr->config |= (1ULL<<46);
         }
     }
+    bdestroy(perf_folder);
     return 0;
 }
 
@@ -1037,11 +1058,12 @@ int perfmon_setupCountersThread_perfevent(
 #if defined(__ARM_ARCH_8A)
                 if (cpuid_info.vendor == FUJITSU_ARM && cpuid_info.part == FUJITSU_A64FX)
                 {
-                    if (event->eventId == 0x3E8 ||
-                        event->eventId == 0x3E0 ||
-                        event->eventId == 0x308 ||
-                        event->eventId == 0x309 ||
-                        (event->eventId >= 0x314 &&  event->eventId <= 0x31E))
+                    if (event->eventId == 0x308 || event->eventId == 0x309 ||
+                        (event->eventId >= 0x314 && event->eventId <= 0x316) ||
+                        (event->eventId >= 0x318 && event->eventId <= 0x31E) ||
+                        event->eventId == 0x330 || event->eventId == 0x350 ||
+                        event->eventId == 0x370 || event->eventId == 0x396 ||
+                        event->eventId == 0x3E0 || event->eventId == 0x3E8)
                     {
                         if (numa_lock[affinity_thread2numa_lookup[cpu_id]] != cpu_id)
                         {
@@ -1115,6 +1137,7 @@ int perfmon_setupCountersThread_perfevent(
                 is_uncore = 1;
                 break;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,5,0)
+            case UNCORE:
             case MBOX0:
             case MBOX1:
             case MBOX2:
@@ -1123,6 +1146,14 @@ int perfmon_setupCountersThread_perfevent(
             case MBOX5:
             case MBOX6:
             case MBOX7:
+            case MBOX8:
+            case MBOX9:
+            case MBOX10:
+            case MBOX11:
+            case MBOX12:
+            case MBOX13:
+            case MBOX14:
+            case MBOX15:
             case CBOX0:
             case CBOX1:
             case CBOX2:
@@ -1392,6 +1423,11 @@ int perfmon_setupCountersThread_perfevent(
                         has_lock = 1;
                     }
                 }
+                if ((cpuid_info.family == ARMV8_FAMILY) && (cpuid_info.part == NVIDIA_GRACE) && cpuid_topology.numSockets > 1)
+                {
+                    DEBUG_PRINT(DEBUGLEV_DEVELOP, Updating uncore type for socket %d on Nvidia Grace, affinity_thread2socket_lookup[cpu_id]);
+                    type += affinity_thread2socket_lookup[cpu_id];
+                }
                 if (has_lock)
                 {
                     ret = perf_uncore_setup(&attr, type, event);
@@ -1516,8 +1552,11 @@ int perfmon_stopCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
                 if (cpuid_info.vendor == FUJITSU_ARM && cpuid_info.part == FUJITSU_A64FX)
                 {
                     switch (eventSet->events[i].event.eventId) {
-                        case 0x3E8:
-                            tmp *= 256;
+                        case 0x1e0:
+                            if (cpuid_topology.numCoresPerSocket == 24)
+                                tmp *= 9;
+                            else
+                                tmp *= 8;
                             break;
                         case 0x3E0:
                             if (cpuid_topology.numCoresPerSocket == 24)
@@ -1525,6 +1564,9 @@ int perfmon_stopCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
                             else
                                 tmp *= 32;
                             break;
+                        case 0x3E8:
+                            tmp *= 256;
+                            break;
                         default:
                             break;
                     }
@@ -1567,8 +1609,11 @@ int perfmon_readCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
                 if (cpuid_info.vendor == FUJITSU_ARM && cpuid_info.part == FUJITSU_A64FX)
                 {
                     switch (eventSet->events[i].event.eventId) {
-                        case 0x3E8:
-                            tmp *= 256;
+                        case 0x1e0:
+                            if (cpuid_topology.numCoresPerSocket == 24)
+                                tmp *= 9;
+                            else
+                                tmp *= 8;
                             break;
                         case 0x3E0:
                             if (cpuid_topology.numCoresPerSocket == 24)
@@ -1576,6 +1621,9 @@ int perfmon_readCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
                             else
                                 tmp *= 32;
                             break;
+                        case 0x3E8:
+                            tmp *= 256;
+                            break;
                         default:
                             break;
                     }
