@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of perfmon module for Intel Sapphire Rapids.
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,13 +38,13 @@ static int perfmon_numCoreCountersSapphireRapids = NUM_COUNTERS_CORE_SAPPHIRERAP
 static int perfmon_numArchEventsSapphireRapids = NUM_ARCH_EVENTS_SAPPHIRERAPIDS;
 
 #define SPR_CHECK_CORE_OVERFLOW(offset) \
-    if (counter_result < data->counterData) \
+    if (counter_result < data[thread_id].counterData) \
     { \
         uint64_t ovf_values = 0x0ULL; \
         CHECK_MSR_READ_ERROR(HPMread(cpu_id, MSR_DEV, MSR_PERF_GLOBAL_STATUS, &ovf_values)); \
         if (ovf_values & (1ULL<<(offset))) \
         { \
-            data->overflows++; \
+            data[thread_id].overflows++; \
         } \
         CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_PERF_GLOBAL_OVF_CTRL, (1ULL<<(offset)))); \
     }
@@ -217,8 +217,8 @@ uint64_t spr_power_start(int thread_id, RegisterIndex index, PerfmonEvent* event
         uint64_t tmp = 0x0ULL;
         RegisterType type = counter_map[index].type;
         CHECK_POWER_READ_ERROR(power_read(cpu_id, counter1,(uint32_t*)&tmp));
-        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST tmp, START_POWER)
         data[thread_id].startData = field64(tmp, 0, box_map[type].regWidth);
+        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST data[thread_id].startData, START_POWER)
     }
     return 0;
 }
@@ -420,7 +420,7 @@ int spr_start_uncore_freerun(int thread_id, RegisterIndex index, PerfmonEvent* e
     int cpu_id = groupSet->threads[thread_id].processorId;
     uint64_t counter1 = counter_map[index].counterRegister;
     PciDeviceIndex dev = counter_map[index].device;
-    data->startData = data->counterData = 0x0;
+    data[thread_id].startData = data[thread_id].counterData = 0x0;
     VERBOSEPRINTPCIREG(cpu_id, dev, counter1, LLU_CAST 0x0ULL, START_UNCORE_FREERUN);
     int err = HPMread(cpu_id, dev, counter1, &flags);
     if (err == 0)
@@ -842,8 +842,8 @@ int perfmon_startCountersThread_sapphirerapids(int thread_id, PerfmonEventSet* e
             uint64_t counter1 = counter_map[index].counterRegister;
 
             PciDeviceIndex dev = counter_map[index].device;
-            data->startData = 0;
-            data->counterData = 0;
+            data[thread_id].startData = 0;
+            data[thread_id].counterData = 0;
             switch (type)
             {
                 case FIXED:
@@ -856,7 +856,7 @@ int perfmon_startCountersThread_sapphirerapids(int thread_id, PerfmonEventSet* e
                     spr_power_start(thread_id, index, event, data);
                     break;
                 case METRICS:
-                    spr_metrics_start(thread_id, index, event, data);
+                    flags |= spr_metrics_start(thread_id, index, event, data);
                     break;
                 case THERMAL:
                 case VOLTAGE:
@@ -1165,7 +1165,7 @@ int perfmon_startCountersThread_sapphirerapids(int thread_id, PerfmonEventSet* e
                 default:
                     break;
             }
-            data->counterData = data->startData;
+            data[thread_id].counterData = data[thread_id].startData;
         }
     }
 
@@ -1231,7 +1231,7 @@ uint32_t spr_power_stop(int thread_id, RegisterIndex index, PerfmonEvent* event,
         uint64_t counter1 = counter_map[index].counterRegister;
         CHECK_POWER_READ_ERROR(power_read(cpu_id, counter1, (uint32_t*)&counter_result));
         VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, STOP_POWER)
-        if (counter_result < data->counterData)
+        if (counter_result < data[thread_id].counterData)
         {
             VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, OVERFLOW_POWER)
             data[thread_id].overflows++;
@@ -1713,7 +1713,7 @@ uint32_t spr_power_read(int thread_id, RegisterIndex index, PerfmonEvent* event,
         uint64_t counter1 = counter_map[index].counterRegister;
         CHECK_POWER_READ_ERROR(power_read(cpu_id, counter1, (uint32_t*)&counter_result));
         VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_POWER)
-        if (counter_result < data->counterData)
+        if (counter_result < data[thread_id].counterData)
         {
             VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, OVERFLOW_POWER)
             data[thread_id].overflows++;
@@ -2171,7 +2171,7 @@ int perfmon_readCountersThread_sapphirerapids(int thread_id, PerfmonEventSet* ev
             }
         }
         VERBOSEPRINTPCIREG(cpu_id, MSR_UBOX_DEVICE, FAKE_UNC_GLOBAL_CTRL, LLU_CAST 0x0ULL, UNFREEZE_UNCORE);
-        HPMwrite(cpu_id, MSR_UBOX_DEVICE, FAKE_UNC_GLOBAL_CTRL, 0x0ULL);
+        CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_UBOX_DEVICE, FAKE_UNC_GLOBAL_CTRL, 0x0ULL));
     }
     if (MEASURE_CORE(eventSet))
     {
@@ -2214,14 +2214,14 @@ int perfmon_finalizeCountersThread_sapphirerapids(int thread_id, PerfmonEventSet
             case FIXED:
                 ovf_values_core |= (1ULL<<(index+32));
                 break;
+            case PMC:
+                ovf_values_core |= (1ULL<<(getCounterTypeOffset(index)));
+                break;
             default:
                 break;
         }
-        if ((reg) && (((type == PMC)||(type == FIXED))||(type == METRICS)|| ((type >= UNCORE) && (haveLock))))
+        if ((reg) && (((type == PMC)||(type == FIXED))||(type == METRICS)|| ((type >= UNCORE && type < NUM_UNITS) && (haveLock))))
         {
-            CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, reg, &ovf_values_uncore));
-            VERBOSEPRINTPCIREG(cpu_id, dev, reg, ovf_values_uncore, SHOW_CTL);
-            ovf_values_uncore = 0x0ULL;
             VERBOSEPRINTPCIREG(cpu_id, dev, reg, 0x0ULL, CLEAR_CTL);
             CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, dev, reg, 0x0ULL));
             if ((type >= SBOX0) && (type <= SBOX3))
