@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of topology module.
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -48,10 +48,10 @@
 #define MAX_MODEL_STRING_LENGTH 512
 
 struct topology_functions {
-    void (*init_cpuInfo) (cpu_set_t cpuSet);
-    void (*init_cpuFeatures) (void);
-    void (*init_nodeTopology) (cpu_set_t cpuSet);
-    void (*init_cacheTopology) (void);
+    int (*init_cpuInfo) (cpu_set_t cpuSet);
+    int (*init_cpuFeatures) (void);
+    int (*init_nodeTopology) (cpu_set_t cpuSet);
+    int (*init_cacheTopology) (void);
     void (*init_fileTopology) (FILE*);
     void (*close_topology) (void);
 };
@@ -117,6 +117,8 @@ struct topology_functions {
 #define ALDERLAKE1           0x97U
 #define ALDERLAKE2           0x9AU
 #define ALDERLAKE3           0xBFU
+#define GRANITERAPIDS        0xADU
+#define SIERRAFORREST        0xAFU
 
 /* Intel MIC */
 #define XEON_PHI           0x01U
@@ -154,7 +156,10 @@ struct topology_functions {
 #define ZEN3_RYZEN3     0x50
 #define ZEN3_EPYC_TRENTO 0x30
 #define ZEN4_RYZEN      0x61
+#define ZEN4_RYZEN_PRO  0x08
+#define ZEN4_RYZEN2     0x74
 #define ZEN4_EPYC       0x11
+#define ZEN4_EPYC_BERGAMO 0xA0
 
 /* ARM */
 #define  ARM7L          0x3U
@@ -164,6 +169,7 @@ struct topology_functions {
 #define  ARM_CORTEX_A57     0xD07U
 #define  ARM_CORTEX_A72     0xD08U
 #define  ARM_CORTEX_A73     0xD09U
+#define  ARM_CORTEX_A76     0xD0BU
 #define  CAV_THUNDERX	0x0A0U
 #define  CAV_THUNDERX88	0x0A1U
 #define  CAV_THUNDERX81	0x0A2U
@@ -178,6 +184,7 @@ struct topology_functions {
 #define  APPLE_M1_STUDIO 0x02U
 #define  HUAWEI_TSV110 0xD01U
 #define  AWS_GRAVITON3 0xD40U
+#define  NVIDIA_GRACE 0xD4FU
 
 /* ARM vendors */
 #define DEFAULT_ARM	0x41U
