@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of topology backend using the hwloc library
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -46,10 +46,10 @@ extern hwloc_topology_t hwloc_topology;
 
 int likwid_hwloc_record_objs_of_type_below_obj(hwloc_topology_t t, hwloc_obj_t obj, hwloc_obj_type_t type, int* index, uint32_t **list);
 
-void hwloc_init_cpuInfo(cpu_set_t cpuSet);
-void hwloc_init_cpuFeatures(void);
-void hwloc_init_nodeTopology(cpu_set_t cpuSet);
-void hwloc_init_cacheTopology(void);
+int hwloc_init_cpuInfo(cpu_set_t cpuSet);
+int hwloc_init_cpuFeatures(void);
+int hwloc_init_nodeTopology(cpu_set_t cpuSet);
+int hwloc_init_cacheTopology(void);
 void hwloc_close(void);
 
 #endif /* TOPOLOGY_HWLOC_H */
