@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of topology backend using procfs/sysfs
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -41,9 +41,9 @@
 #include <bitUtil.h>
 #include <topology.h>
 
-void proc_init_cpuInfo(cpu_set_t cpuSet);
-void proc_init_cpuFeatures(void);
-void proc_init_nodeTopology(cpu_set_t cpuSet);
-void proc_init_cacheTopology(void);
+int proc_init_cpuInfo(cpu_set_t cpuSet);
+int proc_init_cpuFeatures(void);
+int proc_init_nodeTopology(cpu_set_t cpuSet);
+int proc_init_cacheTopology(void);
 
 #endif /* TOPOLOGY_PROC_H */
