@@ -1,3 +1,34 @@
+/*
+ * =======================================================================================
+ *
+ *      Filename:  intel_perfmon_uncore_discovery.c
+ *
+ *      Description:  Code to look up Uncore perfmon units on Intel SPR and later.
+ *
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
+ *
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
+ *      Project:  likwid
+ *
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
+ *
+ *      This program is free software: you can redistribute it and/or modify it under
+ *      the terms of the GNU General Public License as published by the Free Software
+ *      Foundation, either version 3 of the License, or (at your option) any later
+ *      version.
+ *
+ *      This program is distributed in the hope that it will be useful, but WITHOUT ANY
+ *      WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+ *      PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+ *
+ *      You should have received a copy of the GNU General Public License along with
+ *      this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * =======================================================================================
+ */
+
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -6,11 +37,15 @@
 #include <sys/mman.h>
 #include <fcntl.h>
 #include <string.h>
+#include <dirent.h>
 
-
-#include <error.h>
-#include <pci_types.h>
+#include <likwid.h>
 #include <intel_perfmon_uncore_discovery.h>
+#include <perfmon_sapphirerapids_counters.h>
+#include <perfmon_graniterapids_counters.h>
+#include <perfmon_sierraforrest_counters.h>
+
+static PerfmonUncoreDiscovery* uncore_discovery_map = NULL;
 
 
 /* Functions for the same handling of PCI devices and their memory in user-space as in kernel-space */
@@ -23,47 +58,32 @@ struct pci_dev {
     int numa_node;
 };
 
-static void mymemcpy(void* dest, void* src, int size)
+// glibc memcpy uses optimized versions but copies have to be bytewise
+// otherwise the data is incorrect. 
+void mymemcpy(void* dest, void* src, int size)
 {
-    uint8_t* udest = (uint8_t*)dest;
-    uint8_t* usrc = (uint8_t*)src;
+    uint8_t volatile* udest = (uint8_t*)dest;
+    uint8_t volatile* usrc = (uint8_t*)src;
     for (int i = 0; i < size/sizeof(uint8_t); i++)
     {
         *(udest + i) = *(usrc + i);
     }
 }
 
-
-static int pci_test_bus(int domain, int bus)
-{
-    char fname[1025];
-    int ret = snprintf(fname, 1024, "/sys/class/pci_bus/%.04x:%.02x", domain, bus);
-    if (ret > 0)
-    {
-        fname[ret] = '\0';
-    }
-    else
-    {
-        ERROR_PRINT(Cannot format path to PCI bus directory for domain %d and bus %d, domain, bus);
-    }
-    return !access(fname, R_OK);
-}
-
-static int pci_read_config_dword(const struct pci_dev *dev, int where, uint32_t *val)
+// Read 32bit from PCI device config space
+static int pci_read_config_dword(const struct pci_dev *dev, int where, u32 *val)
 {
     int ret = -1;
     char path[1025];
     ret = snprintf(path, 1024, "/sys/bus/pci/devices/%.04x:%.02x:%.02x.%.01x/config", dev->domain, dev->bus, dev->device, dev->func);
     if (ret > 0) {
         path[ret] = '\0';
-        ret = -1;
         if (!access(path, R_OK))
         {
-            //printf("%s\n", path);
             int fp = open(path, O_RDONLY);
             if (fp > 0)
             {
-                ret = pread(fp, val, sizeof(uint32_t), where);
+                ret = pread(fp, val, sizeof(u32), where);
                 close(fp);
             }
         }
@@ -74,13 +94,13 @@ static int pci_read_config_dword(const struct pci_dev *dev, int where, uint32_t
 static int pci_find_next_ext_capability(struct pci_dev *dev, int start, int cap) {
     int ret = 0;
     int offset = PCI_EXT_CAP_BASE_OFFSET + start;
-    uint32_t cap_data = 0x0;
+    u32 cap_data = 0x0;
     do
     {
         ret = pci_read_config_dword(dev, offset, &cap_data);
         if (ret > 0)
         {
-            uint16_t cap_id = (cap_data & PCI_EXT_CAP_ID_MASK);
+            u16 cap_id = (cap_data & PCI_EXT_CAP_ID_MASK);
             if (cap_id == cap) break;
             ret = pci_read_config_dword(dev, offset + PCI_EXT_CAP_NEXT_OFFSET, &cap_data);
             offset = (ret > 0 ? (cap_data >> PCI_EXT_CAP_NEXT_SHIFT & PCI_EXT_CAP_NEXT_MASK) : 0);
@@ -91,6 +111,7 @@ static int pci_find_next_ext_capability(struct pci_dev *dev, int start, int cap)
     return offset;
 }
 
+
 /* Helper function to read files from sysfs */
 static int file_to_uint(char* path, unsigned int *data) {
     int ret = -1;
@@ -101,7 +122,7 @@ static int file_to_uint(char* path, unsigned int *data) {
         ret = read(fp, content, 1024 * sizeof(char));
         if (ret >= 0) {
             content[ret] = '\0';
-            unsigned int new = 0x0;
+            u64 new = 0x0;
             ret = sscanf(content, "0x%x", &new);
             if (ret == 1)
             {
@@ -124,9 +145,90 @@ static int file_to_uint(char* path, unsigned int *data) {
     return ret;
 }
 
-
-struct pci_dev * pci_get_device(unsigned int vendor, unsigned int device, struct pci_dev * from)
+// Test whether PCI domain and bus exist.
+static int pci_test_bus(int domain, int bus)
 {
+    char fname[1025];
+    int ret = snprintf(fname, 1024, "/sys/class/pci_bus/%.04x:%.02x", domain, bus);
+    if (ret > 0)
+    {
+        fname[ret] = '\0';
+    }
+    else
+    {
+        ERROR_PRINT(Cannot format path to PCI bus directory for domain %d and bus %d, domain, bus);
+    }
+    return !access(fname, R_OK);
+}
+
+// commonly only a few PCI domains are populated, so we get the max domain
+// to return from PCI device parsing early. Otherwise, we have to search
+// 65535 domains.
+static int get_max_pci_domain()
+{
+    int max_dom = 0;
+    DIR* dir = NULL;
+    struct dirent* dent = NULL;
+
+    dir = opendir("/sys/class/pci_bus");
+    if (!dir)
+    {
+        return -errno;
+    }
+    while((dent = readdir(dir)))
+    {
+        int dom = -1, bus = -1;
+        int c = sscanf(dent->d_name, "%X:%X", &dom, &bus);
+        if ((c == 2) && (dom > max_dom))
+        {
+            max_dom = dom;
+        }
+    }
+
+    closedir(dir);
+    return max_dom+1;
+}
+
+// Get the maximal CPU socket ID to stop parsing. This does not work if
+// sockets are not numbered consecutively. 
+static int max_socket_id(int* max_socket)
+{
+    int num_procs = sysconf (_SC_NPROCESSORS_CONF);
+    int cur_procs = 0;
+    int ret = -1;
+    int cur_max = 0;
+    char path[1025];
+    for (int i = 0; i < 100000 && cur_procs < num_procs; i++)
+    {
+        ret = snprintf(path, 1024, "/sys/devices/system/cpu/cpu%d/topology/physical_package_id", i);
+        if (ret >= 0)
+        {
+            path[ret] = '\0';
+            int id = 0;
+            ret = file_to_uint(path, &id);
+            if (ret == 0 && id > cur_max)
+            {
+                cur_max = id;
+            }
+        }
+    }
+    DEBUG_PRINT(DEBUGLEV_DEVELOP, Found max socket ID %d, cur_max);
+    *max_socket = cur_max;
+    return 0;
+}
+
+
+
+
+static int _max_pci_domain_id = -1;
+// This functions should mimic the behavior of pci_get_device in the kernel
+// It searches the whole PCI space for a specific device and defined by vendor
+// and device ID. Instead of vendor and device ID, the special PCI_ANY_ID can
+// be used to get any device. In order to not start from zero each time, the
+// function takes a device as input (from) to start from there.
+struct pci_dev * pci_get_device (unsigned int vendor, unsigned int device, struct pci_dev * from)
+{
+    //struct pci_dev next = {from->domain, from->bus, from->device, from->func};
     int domStart = (from ? from->domain : 0);
     int busStart = (from ? from->bus : 0);
     int devStart = (from ? from->device : 0);
@@ -134,24 +236,38 @@ struct pci_dev * pci_get_device(unsigned int vendor, unsigned int device, struct
     char busbase[1025];
     char devbase[1025];
     char fname[1025];
-    if (from)
+    
+    if (_max_pci_domain_id < 0)
     {
-        domStart = from->domain;
-        busStart = from->bus;
-        devStart = from->device;
-        funcStart = from->func;
+        int ret = get_max_pci_domain();
+        if (ret < 0)
+        {
+            return NULL;
+        }
+        _max_pci_domain_id = ret;
     }
-    for (int dom = domStart; dom < 0xffff; dom++) {
+
+    for (int dom = domStart; dom < _max_pci_domain_id; dom++) {
+        // if we are beyond the domain given in from, we start with bus=0 again
+        // instead of the bus given in from
+        if (from && dom > from->domain) busStart = 0;
         for (int bus = busStart; bus < 0xff; bus++) {
             int ret = 0;
             // Early skip if PCI bus does not exist
             if (!pci_test_bus(dom, bus)) {
+            
                 continue;
             }
+            // if we are beyond the bus given in from, we start with devid=0 again
+            // instead of the device given in from
+            if (from && bus > from->bus) devStart = 0;
             for (int devid = devStart; devid < 0xff; devid++) {
+                // if we are beyond the device given in from, we start with func=0 again
+                // instead of the func given in from
+                if (from && devid > from->device) funcStart = 0;
                 for (int func = funcStart; func < 0xf; func++) {
                     // Skip the 'from' device
-                    if (from && dom == domStart && bus == busStart && devid == devStart && func == funcStart)
+                    if (from && dom == from->domain && bus == from->bus && devid == from->device && func ==  from->func)
                     {
                         continue;
                     }
@@ -222,7 +338,6 @@ struct pci_dev * pci_get_device(unsigned int vendor, unsigned int device, struct
                     // Reuse space
                     if (from)
                     {
-                       DEBUG_PRINT(DEBUGLEV_DEVELOP, Using discovery entry device %s, devbase);
                         from->domain = dom;
                         from->bus = bus;
                         from->device = devid;
@@ -235,7 +350,6 @@ struct pci_dev * pci_get_device(unsigned int vendor, unsigned int device, struct
                         // First find, allocate a new PCI device
                         struct pci_dev* next = malloc(sizeof(struct pci_dev));
                         if (next) {
-                           DEBUG_PRINT(DEBUGLEV_DEVELOP, Using discovery entry device %s, devbase);
                             next->domain = dom;
                             next->bus = bus;
                             next->device = devid;
@@ -243,10 +357,6 @@ struct pci_dev * pci_get_device(unsigned int vendor, unsigned int device, struct
                             next->numa_node = node;
                             return next;
                         }
-                       else
-                       {
-                           ERROR_PRINT(Failed to allocate space for PCI device info);
-                       }
                     }
                 }
             }
@@ -255,172 +365,113 @@ struct pci_dev * pci_get_device(unsigned int vendor, unsigned int device, struct
     return NULL;
 }
 
+
 PciDeviceIndex get_likwid_device(int type, int id)
 {
-    //printf("get_likwid_device(%d, %d) = ", type, id);
-    switch (type)
+    int i = 0;
+    while (uncore_discovery_map[i].discovery_type >= 0)
     {
-        case DEVICE_ID_CHA:
-            if (id < 0 || id > 60)
+        if (uncore_discovery_map[i].discovery_type == type)
+        {
+            if (id >= 0 && id < uncore_discovery_map[i].max_devices)
             {
-                ERROR_PRINT(Cannot transform CHa device with ID %d, id);
+                return uncore_discovery_map[i].base_device + id;
             }
-            else
-            {
-                return MSR_CBOX_DEVICE_C0 + id;
-            }
-            break;
-        case DEVICE_ID_iMC:
-            if (id < 0 || id > 15)
-            {
-                ERROR_PRINT(Cannot transform IMC device with ID %d, id);
-            }
-            else
-            {
-                return MMIO_IMC_DEVICE_0_CH_0 + id;
-            }
-            break;
-        case DEVICE_ID_M2PCIe:
-            if (id < 0 || id > 15)
-            {
-                ERROR_PRINT(Cannot transform M2PCIe device with ID %d, id);
-            }
-            else
-            {
-                return PCI_R2PCIE_DEVICE0 + id;
-            }
-            break;
-        case DEVICE_ID_PCU:
-            if (id < 0 || id > 1)
-            {
-                ERROR_PRINT(Cannot transform PCU device with ID %d, id);
-            }
-            else
-            {
-                return MSR_PCU_DEVICE;
-            }
-            break;
-        case DEVICE_ID_IRP:
-            if (id >= 0 && id <= 12)
-            {
-                return MSR_IRP_DEVICE_0 + id;
-            }
-            else
-            {
-                ERROR_PRINT(Cannot transform IRP device with ID %d, id);
-            }
-            break;
-        case DEVICE_ID_IIO:
-            if (id >= 0 && id <= 12)
-            {
-                return MSR_IIO_DEVICE_0 + id;
-            }
-            else
-            {
-                ERROR_PRINT(Cannot transform IIO device with ID %d, id);
-            }
-            break;
-        case DEVICE_ID_UPI:
-            if (id >= 0 && id <= 3)
-            {
-                return PCI_QPI_DEVICE_PORT_0 + id;
-            }
-            else
-            {
-                ERROR_PRINT(Cannot transform UPI device with ID %d, id);
-            }
-            break;
-        case DEVICE_ID_MDF:
-            if (id < 0 || id > 49)
-            {
-                ERROR_PRINT(Cannot transform MDF device with ID %d, id);
-            }
-            else
-            {
-                return MSR_MDF_DEVICE_0 + id;
-            }
-            break;
-        case DEVICE_ID_M2M:
-            if (id < 0 || id > 31)
-            {
-                ERROR_PRINT(Cannot transform M2M device with ID %d, id);
-                return MAX_NUM_PCI_DEVICES;
-            }
-            else
-            {
-                return PCI_HA_DEVICE_0 + id;
-            }
-            break;
-        case DEVICE_ID_M3UPI:
-            if (id < 0 || id > 3)
-            {
-                ERROR_PRINT(Cannot transform M3UPI device with ID %d, id);
-            }
-            else
-            {
-                return PCI_R3QPI_DEVICE_LINK_0 + id;
-            }
-            break;
-        case DEVICE_ID_HBM:
-            if (id < 0 || id > 31)
-            {
-                ERROR_PRINT(Cannot transform HBM device with ID %d, id);
-            }
-            else
-            {
-                return MMIO_HBM_DEVICE_0 + id;
-            }
-            break;
-        default:
-            return MAX_NUM_PCI_DEVICES;
-            break;
+        }
+        i++;
     }
+    return MAX_NUM_PCI_DEVICES;
 }
 
 
 static void print_unit(PciDeviceIndex idx, PerfmonDiscoveryUnit* unit)
 {
-    DEBUG_PRINT(DEBUGLEV_DEVELOP, PCIIDX %d Access %s NumRegs %d ID %d Type %s(%d) box_ctl 0x%X ctrl_offset 0x%X ctr_offset 0x%X mmap_addr 0x%X mmap_offset 0x%X, idx, AccessTypeNames[unit->access_type], unit->num_regs, unit->box_id, uncore_discovery_box_type_names[unit->box_type], unit->box_type, unit->box_ctl, unit->ctrl_offset, unit->ctr_offset, unit->mmap_addr, unit->mmap_offset);
+    char* name = NULL;
+    int i = 0;
+    if (!uncore_discovery_map) return;
+
+    while (uncore_discovery_map[i].discovery_type >= 0)
+    {
+        if (unit->box_type == uncore_discovery_map[i].discovery_type)
+        {
+            name = uncore_discovery_map[i].name;
+            break;
+        }
+        i++;
+    }
+    if (name != NULL)
+    {
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, PCIIDX %d Access %s NumRegs %d ID %d Type %s(%d) box_ctl 0x%X ctrl_offset 0x%X ctr_offset 0x%X mmap_addr 0x%X mmap_offset 0x%X,
+                idx, AccessTypeNames[unit->access_type], unit->num_regs, unit->box_id, name, unit->box_type,
+                unit->box_ctl, unit->ctrl_offset, unit->ctr_offset, unit->mmap_addr, unit->mmap_offset);
+    }
 }
 
-int max_socket_id(int* max_socket)
+static int perfmon_uncore_discovery_update_dev_location(PerfmonDiscoveryUnit* unit)
 {
-    int num_procs = sysconf (_SC_NPROCESSORS_CONF);
-    int cur_procs = 0;
-    int ret = -1;
-    int cur_max = 0;
-    char path[1025];
-    for (int i = 0; i < 100000 && cur_procs < num_procs; i++)
+    struct pci_dev* dev = NULL;
+    uint32_t device = 0;
+    uint32_t devfn = 0;
+    switch (unit->box_type)
     {
-        ret = snprintf(path, 1024, "/sys/devices/system/cpu/cpu%d/topology/physical_package_id", i);
-        if (ret >= 0)
+        case SPR_DEVICE_ID_UPI:
+            device = 0x3241;
+            devfn = 0x9;
+            break;
+        case SPR_DEVICE_ID_M3UPI:
+            device = 0x3246;
+            devfn = 0x29;
+            break;
+        default:
+            return 0;
+    }
+
+    while ((dev = pci_get_device(PCI_VENDOR_ID_INTEL, device, dev)) != NULL)
+    {
+        if (dev->numa_node < 0)
         {
-            path[ret] = '\0';
-            int id = 0;
-            ret = file_to_uint(path, &id);
-            if (ret == 0 && id > cur_max)
-            {
-                cur_max = id;
-            }
+            continue;
         }
+        unit->box_ctl = dev->domain  << UNCORE_DISCOVERY_PCI_DOMAIN_OFFSET |
+                        dev->bus << UNCORE_DISCOVERY_PCI_BUS_OFFSET |
+                        devfn << UNCORE_DISCOVERY_PCI_DEVFN_OFFSET |
+                        unit->box_ctl;
     }
-    DEBUG_PRINT(DEBUGLEV_DEVELOP, Found max socket ID %d, cur_max);
-    *max_socket = cur_max;
     return 0;
 }
 
-int perfmon_uncore_discovery(PerfmonDiscovery** perfmon)
+int perfmon_uncore_discovery(int model, PerfmonDiscovery** perfmon)
 {
-    int num = 0;
     int ret = 0;
     PerfmonDiscoverySocket* socks = NULL;
-    int socket_id = -1;
+    int socket_id = 0;
     struct pci_dev* dev = NULL;
     struct uncore_global_discovery global;
     int dvsec = 0;
     int PAGE_SIZE = sysconf (_SC_PAGESIZE);
-    void* io_addr = NULL;
+    void * io_addr = NULL;
     int max_sockets = 0;
+    u32 val = 0;
+    u32 entry_id = 0;
+    u32 bir;
+    u32 bar_offset = 0, pci_dword = 0;
+    u64 addr = 0;
+
+    switch (model)
+    {
+        case SAPPHIRERAPIDS:
+            uncore_discovery_map = sapphirerapids_uncore_discovery_map;
+            break;
+        case GRANITERAPIDS:
+            uncore_discovery_map = graniterapids_uncore_discovery_map;
+            break;
+        case SIERRAFORREST:
+            uncore_discovery_map = sierraforrest_uncore_discovery_map;
+            break;
+        default:
+            ERROR_PRINT(Uncore discovery not supported for model 0x%X, model);
+            return -1;
+    }
 
     if (max_socket_id(&max_sockets) < 0)
     {
@@ -445,63 +496,81 @@ int perfmon_uncore_discovery(PerfmonDiscovery** perfmon)
     perf->sockets = NULL;
     perf->num_sockets = 0;
 
-    while (((dev = pci_get_device(0x8086, UNCORE_DISCOVERY_TABLE_DEVICE, dev)) != NULL)  && (socket_id < max_sockets)){
-        socket_id++;
-
-        PerfmonDiscoverySocket* tmp = realloc(perf->sockets, (num + 1) * sizeof(PerfmonDiscoverySocket));
-        if (!tmp)
-        {
-            ERROR_PRINT(Cannot enlarge socket device table to %d, num);
-            if (perf->sockets) free(perf->sockets);
-            free(perf);
-            close(pcihandle);
-            return -ENOMEM;
-        }
-        perf->sockets = tmp;
-        PerfmonDiscoverySocket* cur = &perf->sockets[num];
-
-        cur->socket_id = socket_id;
-        memset(cur->units, 0, MAX_NUM_PCI_DEVICES*sizeof(PerfmonDiscoveryUnit));
-
+    while (((dev = pci_get_device(PCI_VENDOR_ID_INTEL, PCI_ANY_ID, dev)) != NULL)  && (socket_id < max_sockets+1)){
         while ((dvsec = pci_find_next_ext_capability(dev, dvsec, UNCORE_EXT_CAP_ID_DISCOVERY))) {
             /* read the DVSEC_ID (15:0) */
-            uint32_t val = 0;
+            val = 0;
             ret = pci_read_config_dword(dev, dvsec + UNCORE_DISCOVERY_DVSEC_OFFSET, &val);
-            if (ret < 0) continue;
-            uint32_t entry_id = val & UNCORE_DISCOVERY_DVSEC_ID_MASK;
-            if (entry_id == UNCORE_DISCOVERY_DVSEC_ID_PMON) {
-                uint32_t bir = 0;
+            if (ret < 0)
+            {
+                ERROR_PRINT(Failed to read DVSEC offset from device %.04x:%.02x:%.02x.%.01x, dev->domain, dev->bus, dev->device, dev->func);
+                continue;
+            }
+            entry_id = val & UNCORE_DISCOVERY_DVSEC_ID_MASK;
+            if (entry_id == UNCORE_DISCOVERY_DVSEC_ID_PMON)
+            {
+                //printf("Socket %d Dev %.04x:%.02x:%.02x.%.01x Node %d\n", socket_id, dev->domain, dev->bus, dev->device, dev->func, dev->numa_node);
+                bir = 0;
                 ret = pci_read_config_dword(dev, dvsec + UNCORE_DISCOVERY_DVSEC2_OFFSET, &bir);
-                if (ret < 0) continue;
-                /* read BIR value (2:0) */
+                if (ret < 0)
+                {
+                    ERROR_PRINT(Failed to read DIR from device %.04x:%.02x:%.02x.%.01x, dev->domain, dev->bus, dev->device, dev->func);
+                    continue;
+                }
+               /* read BIR value (2:0) */
                 bir = bir & UNCORE_DISCOVERY_DVSEC2_BIR_MASK;
                 /* calculate the BAR offset of global discovery table */
-                uint32_t bar_offset = UNCORE_DISCOVERY_BIR_BASE + (bir * UNCORE_DISCOVERY_BIR_STEP);
+                bar_offset = 0x10 + (bir * 4);
                 /* read the BAR address of global discovery table */
-                uint32_t pci_dword = 0;
                 ret = pci_read_config_dword(dev, bar_offset, &pci_dword);
-                if (ret < 0) continue;
+                if (ret < 0)
+                {
+                    ERROR_PRINT(Failed to read BAR offset from device %.04x:%.02x:%.02x.%.01x, dev->domain, dev->bus, dev->device, dev->func);
+                    continue;
+                }
                 /* get page boundary address of pci_dword */
-                uint64_t addr = pci_dword & (~(PAGE_SIZE - 1));
+                addr = pci_dword & ~(PAGE_SIZE - 1);
                 if ((pci_dword & PCI_BASE_ADDRESS_MEM_TYPE_MASK) == PCI_BASE_ADDRESS_MEM_TYPE_64)
                 {
+                    DEBUG_PRINT(DEBUGLEV_DEVELOP, Mem type 64);
                     uint32_t pci_dword2;
                     ret = pci_read_config_dword(dev, bar_offset + sizeof(uint32_t), &pci_dword2);
                     if (ret > 0) addr |= ((uint64_t)pci_dword2) << 32;
                 }
                 /* Map whole discovery table */
                 /* User-space version of ioremap */
+                //printf("MMap address 0x%lX\n", addr);
                 io_addr = mmap(NULL, UNCORE_DISCOVERY_MAP_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED, pcihandle, addr);
                 if (io_addr == MAP_FAILED)
                 {
+                    ERROR_PRINT(Failed to mmap device %.04x:%.02x:%.02x.%.01x, dev->domain, dev->bus, dev->device, dev->func);
                     continue;
                 }
+                memset(&global, 0, sizeof(struct uncore_global_discovery));
                 mymemcpy(&global, io_addr, sizeof(struct uncore_global_discovery));
                 if (uncore_discovery_invalid_unit(global))
                 {
                     munmap(io_addr, UNCORE_DISCOVERY_MAP_SIZE);
                     continue;
                 }
+                //printf("Global 1=0x%lX 2=0x%lX 3=0x%lX\n", global.table1, global.table2, global.table3);
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, Device %.04x:%.02x:%.02x.%.01x usable with %d units, dev->domain, dev->bus, dev->device, dev->func, global.max_units);
+
+                PerfmonDiscoverySocket* tmp = realloc(perf->sockets, (socket_id + 1) * sizeof(PerfmonDiscoverySocket));
+                if (!tmp)
+                {
+                    ERROR_PRINT(Cannot enlarge socket device table to %d, socket_id);
+                    if (perf->sockets) free(perf->sockets);
+                    free(perf);
+                    close(pcihandle);
+                    munmap(io_addr, UNCORE_DISCOVERY_MAP_SIZE);
+                    return -ENOMEM;
+                }
+                perf->sockets = tmp;
+                PerfmonDiscoverySocket* cur = &perf->sockets[socket_id];
+
+                cur->socket_id = socket_id;
+                memset(cur->units, 0, MAX_NUM_PCI_DEVICES*sizeof(PerfmonDiscoveryUnit));
                 // record stuff from global struct in cur->global
                 cur->global.global_ctl = global.global_ctl;
                 cur->global.access_type = global.access_type;
@@ -510,7 +579,11 @@ int perfmon_uncore_discovery(PerfmonDiscovery** perfmon)
                 for (int i = 0; i < global.max_units; i++)
                 {
                     struct uncore_unit_discovery unit;
-                    if ((i + 1) * (global.stride * 8) > UNCORE_DISCOVERY_MAP_SIZE) continue;
+                    if ((i + 1) * (global.stride * 8) > UNCORE_DISCOVERY_MAP_SIZE)
+                    {
+                        //ERROR_PRINT(Access to 0x%X outside of mapped memory, (i + 1) * (global.stride * 8));
+                        continue;
+                    }
                     mymemcpy(&unit, io_addr + (i + 1) * (global.stride * 8), sizeof(struct uncore_unit_discovery));
                     if ((uncore_discovery_invalid_unit(unit)) || (unit.num_regs == 0))
                     {
@@ -520,7 +593,7 @@ int perfmon_uncore_discovery(PerfmonDiscovery** perfmon)
                     PciDeviceIndex idx = get_likwid_device(unit.box_type, unit.box_id);
                     if (idx >= 0 && idx < MAX_NUM_PCI_DEVICES)
                     {
-                        cur->units[idx].box_type = (uncore_discovery_box_types) unit.box_type;
+                        cur->units[idx].box_type = unit.box_type;
                         cur->units[idx].box_id = unit.box_id;
                         cur->units[idx].num_regs = unit.num_regs;
                         cur->units[idx].ctrl_offset = unit.ctl_offset;
@@ -528,25 +601,35 @@ int perfmon_uncore_discovery(PerfmonDiscovery** perfmon)
                         cur->units[idx].ctr_offset = unit.ctr_offset;
                         cur->units[idx].status_offset = unit.status_offset;
                         cur->units[idx].access_type = (AccessTypes)unit.access_type;
-/*                        if (cur->units[idx].access_type == ACCESS_TYPE_PCI)*/
-/*                        {*/
-/*                            // Workaround: When running the perfmon discovery in kernel-space, the*/
-/*                            // mapped PCI devices differ to the user-space discovery by bit 27 only*/
-/*                            // for the PCI based Uncore devices.*/
-/*                            unit.box_ctl |= (1<<27);*/
-/*                        }*/
                         cur->units[idx].box_ctl = unit.box_ctl;
                         cur->units[idx].filter_offset = 0x0;
                         cur->units[idx].fixed_ctrl_offset = 0x0;
                         cur->units[idx].fixed_ctr_offset = 0x0;
-                        if (unit.box_type == DEVICE_ID_CHA)
+                        switch (model)
                         {
-                            cur->units[idx].filter_offset = 0xE;
+                            case SAPPHIRERAPIDS:
+                            case GRANITERAPIDS:
+                                if (unit.box_type == SPR_DEVICE_ID_CHA)
+                                {
+                                    cur->units[idx].filter_offset = 0xE;
+                                }
+                                else if (unit.box_type == SPR_DEVICE_ID_iMC || (unit.box_type == SPR_DEVICE_ID_HBM && model == SAPPHIRERAPIDS))
+                                {
+                                    cur->units[idx].fixed_ctrl_offset = 0x54;
+                                    cur->units[idx].fixed_ctr_offset = 0x38;
+                                }
+                                break;
+                            default:
+                                break;
                         }
-                        else if (unit.box_type == DEVICE_ID_iMC || unit.box_type == DEVICE_ID_HBM)
+
+                        switch (model)
                         {
-                            cur->units[idx].fixed_ctrl_offset = 0x54;
-                            cur->units[idx].fixed_ctr_offset = 0x38;
+                            case SAPPHIRERAPIDS:
+                                perfmon_uncore_discovery_update_dev_location(&cur->units[idx]);
+                                break;
+                            default:
+                                break;
                         }
 
                         cur->units[idx].mmap_addr = cur->units[idx].box_ctl & ~(PAGE_SIZE - 1);
@@ -558,10 +641,8 @@ int perfmon_uncore_discovery(PerfmonDiscovery** perfmon)
                 }
                 /* Unmap PCI config space */
                 munmap(io_addr, UNCORE_DISCOVERY_MAP_SIZE);
-            }
-            else
-            {
-                DEBUG_PRINT(DEBUGLEV_DEVELOP, not the right dvsec);
+                io_addr = NULL;
+                socket_id++;
             }
         }
 /*        printf("Add M2IOSF devices \n");*/
@@ -583,15 +664,10 @@ int perfmon_uncore_discovery(PerfmonDiscovery** perfmon)
 /*            cur->units[idx].fixed_ctr_offset = 0x0;*/
 /*            print_unit(idx, &cur->units[idx]);*/
 /*        }*/
-        num++;
-    }
-    if (num == 0)
-    {
-         ERROR_PRINT(Failed to get any socket device tables);
-        return -EFAULT;
     }
+    if (dev) free(dev);
     close(pcihandle);
-    perf->num_sockets = num;
+    perf->num_sockets = socket_id+1;
     *perfmon = perf;
     return 0;
 }
@@ -607,7 +683,7 @@ void perfmon_uncore_discovery_free(PerfmonDiscovery* perfmon)
             {
                 if (perfmon->sockets[i].units[j].io_addr)
                 {
-                    munmap(perfmon->sockets[i].units[j].io_addr, perfmon->sockets[i].units[j].mmap_size);
+                    munmap((void*)perfmon->sockets[i].units[j].io_addr, perfmon->sockets[i].units[j].mmap_size);
                     perfmon->sockets[i].units[j].io_addr = NULL;
                 }
             }
