@@ -5,14 +5,14 @@
  *
  *      Description:  Marker API interface of module perfmon
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -159,6 +159,7 @@ likwid_markerInit(void)
     bstring bEventStr;
     struct bstrList* threadTokens;
     struct bstrList* eventStrings;
+    cpu_set_t cpuset;
     char* modeStr = getenv("LIKWID_MODE");
     char* eventStr = getenv("LIKWID_EVENTS");
     char* cThreadStr = getenv("LIKWID_THREADS");
@@ -191,6 +192,53 @@ likwid_markerInit(void)
         return;
     }
 
+    if (debugStr != NULL)
+    {
+        verbosity = atoi(debugStr);
+        perfmon_setVerbosity(verbosity);
+    }
+
+    bThreadStr = bfromcstr(cThreadStr);
+    threadTokens = bsplit(bThreadStr,',');
+    num_cpus = threadTokens->qty;
+    for (i = 0; i < num_cpus; i++)
+    {
+        threads2Cpu[i] = ownatoi(bdata(threadTokens->entry[i]));
+    }
+    bdestroy(bThreadStr);
+    bstrListDestroy(threadTokens);
+
+    CPU_ZERO(&cpuset);
+    if (pinStr != NULL)
+    {
+        // If LIKWID should pin the threads, we check whether we are already
+        // in a restricted cpuset with a single hwthread. This happens
+        // when an OpenMP region is executed before LIKWID_MARKER_INIT.
+        // When the first thread starts up, LIKWID pins the master process
+        // and the thread. But we want to get the topology without the restriction
+        // here. We don't need to set it back because the first operation
+        // after getting the topology is the pinning of the master process.
+        int err = sched_getaffinity(0, sizeof(cpu_set_t), &cpuset);
+        if (err == 0)
+        {
+            int count = CPU_COUNT(&cpuset);
+            if (count == 1 && count != num_cpus)
+            {
+                cpu_set_t newcpuset;
+                CPU_ZERO(&newcpuset);
+                for (i = 0; i < num_cpus; i++)
+                {
+                    CPU_SET(threads2Cpu[i], &newcpuset);
+                }
+                err = sched_setaffinity(0, sizeof(cpu_set_t), &newcpuset);
+            }
+        }
+        else
+        {
+            fprintf(stderr,"Failed to get cpuset for master thread.\n");
+        }
+    }
+
     topology_init();
     numa_init();
     affinity_init();
@@ -199,21 +247,6 @@ likwid_markerInit(void)
 #ifndef LIKWID_USE_PERFEVENT
     HPMmode(atoi(modeStr));
 #endif
-    if (debugStr != NULL)
-    {
-        perfmon_verbosity = atoi(debugStr);
-        verbosity = perfmon_verbosity;
-    }
-
-    bThreadStr = bfromcstr(cThreadStr);
-    threadTokens = bsplit(bThreadStr,',');
-    num_cpus = threadTokens->qty;
-    for (i=0; i<num_cpus; i++)
-    {
-        threads2Cpu[i] = ownatoi(bdata(threadTokens->entry[i]));
-    }
-    bdestroy(bThreadStr);
-    bstrListDestroy(threadTokens);
 
     if (pinStr != NULL)
     {
@@ -502,6 +535,131 @@ likwid_markerClose(void)
     likwid_init = 0;
 }
 
+int
+likwid_markerWriteFile(const char* markerfile)
+{
+    if (markerfile == NULL)
+    {
+        fprintf(stderr, "File can not be NULL.\n");
+        return -EFAULT;
+    }
+
+    FILE *file = NULL;
+    int* validRegions = NULL;
+
+    int numberOfThreads = perfmon_getNumberOfThreads();
+    int numberOfRegions = perfmon_getNumberOfRegions();
+
+    if ( ! likwid_init )
+    {
+        return -EFAULT;;
+    }
+    if ((numberOfThreads == 0)||(numberOfRegions == 0))
+    {
+        fprintf(stderr, "No threads or regions defined in hash table\n");
+        return -EFAULT;
+    }
+
+    file = fopen(markerfile,"w");
+    if (file != NULL)
+    {
+        int newNumberOfRegions = 0;
+        int newRegionID = 0;
+        validRegions = (int*)malloc(numberOfRegions*sizeof(int));
+        if (!validRegions)
+        {
+            return -EFAULT;
+        }
+        for (int i=0; i<numberOfRegions; i++)
+        {
+            validRegions[i] = 0;
+        }
+        for (int i=0; i<numberOfRegions; i++)
+        {
+            for (int j=0; j<numberOfThreads; j++)
+            {
+                validRegions[i] += perfmon_getCountOfRegion(i, j);
+            }
+            if (validRegions[i] > 0)
+                newNumberOfRegions++;
+            else
+                fprintf(stderr, "WARN: Skipping region %s for evaluation.\n", perfmon_getTagOfRegion(i));
+        }
+        if (newNumberOfRegions < numberOfRegions)
+        {
+            fprintf(stderr, "WARN: Regions are skipped because:\n");
+            fprintf(stderr, "      - The region was only registered\n");
+            fprintf(stderr, "      - The region was started but never stopped\n");
+            fprintf(stderr, "      - The region was never started but stopped\n");
+        }
+        DEBUG_PRINT(DEBUGLEV_DEVELOP,
+                Creating Marker file %s with %d regions %d groups and %d threads,
+                markerfile, newNumberOfRegions, numberOfGroups, numberOfThreads);
+        bstring thread_regs_grps = bformat("%d %d %d", numberOfThreads, newNumberOfRegions, numberOfGroups);
+        fprintf(file,"%s\n", bdata(thread_regs_grps));
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, %s, bdata(thread_regs_grps));
+        bdestroy(thread_regs_grps);
+
+        for (int i=0; i<numberOfRegions; i++)
+        {
+            if (validRegions[i] == 0)
+                continue;
+            bstring tmp = bformat("%d:%s", newRegionID, perfmon_getTagOfRegion(i));
+            fprintf(file,"%s\n", bdata(tmp));
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, %s, bdata(tmp));
+            bdestroy(tmp);
+            newRegionID++;
+        }
+        int *cpulist = (int*) malloc(numberOfThreads * sizeof(int));
+        if (cpulist == NULL)
+        {
+            fprintf(stderr, "Failed to allocate %lu bytes for the cpulist storage\n",
+                        numberOfThreads * sizeof(int));
+            free(validRegions);
+            return -EFAULT;
+        }
+        newRegionID = 0;
+        for (int i=0; i<numberOfRegions; i++)
+        {
+            if (validRegions[i] == 0)
+                continue;
+            int groupID = perfmon_getGroupOfRegion(i);
+            int nevents = groupSet->groups[groupID].numberOfEvents;
+            int cpulist_size = perfmon_getCpulistOfRegion(i, numberOfThreads, cpulist);
+            for (int j=0; j<numberOfThreads; j++)
+            {
+                int count = perfmon_getCountOfRegion(i, j);
+                double time = perfmon_getTimeOfRegion(i, j);
+                bstring l = bformat("%d %d %d %u %e %d ", newRegionID,
+                                                          groupID,
+                                                          cpulist[j],
+                                                          count,
+                                                          time,
+                                                          nevents);
+
+                for (int k=0; k < MIN(nevents, NUM_PMC); k++)
+                {
+                    bstring tmp = bformat("%e ", perfmon_getResultOfRegionThread(i, k, j));
+                    bconcat(l, tmp);
+                    bdestroy(tmp);
+                }
+                fprintf(file,"%s\n", bdata(l));
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, %s, bdata(l));
+                bdestroy(l);
+            }
+            newRegionID++;
+        }
+        fclose(file);
+        free(validRegions);
+        free(cpulist);
+    }
+    else
+    {
+        fprintf(stderr, "Cannot open file %s\n", markerfile);
+        fprintf(stderr, "%s", strerror(errno));
+    }
+}
+
 int
 likwid_markerRegisterRegion(const char* regionTag)
 {
@@ -762,7 +920,7 @@ likwid_markerResetRegion(const char* regionTag)
 }
 
 int
-likwid_getProcessorId()
+likwid_getProcessorId(void)
 {
     int i;
     cpu_set_t  cpu_set;
