@@ -5,14 +5,14 @@
  *
  *      Description: F90 interface for marker API
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com,
  *               Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -56,6 +56,24 @@ likwid_markerclose_(void)
     likwid_markerClose();
 }
 
+void __attribute__ ((visibility ("default") ))
+likwid_markerwritefile_(char* markerfile, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char));
+    strncpy(tmp, markerfile, len * sizeof(char));
+
+    for (int i = (len - 1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i + 1] = 0;
+            break;
+        }
+    }
+
+    likwid_markerWriteFile(tmp);
+    free(tmp);
+}
+
 void __attribute__ ((visibility ("default") ))
 likwid_markernextgroup_(void)
 {
@@ -248,6 +266,23 @@ likwid_nvmarkerresetregion_(char* regionTag, int len)
     nvmon_markerResetRegion( tmp);
     free(tmp);
 }
+
+int __attribute__ ((visibility ("default") ))
+likwid_nvmarkerwritefile_(char* filename, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char));
+    strncpy(tmp, filename, len * sizeof(char) );
+
+    for (int i=(len-1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i+1] = 0;
+            break;
+        }
+    }
+    nvmon_markerWriteFile(tmp);
+    free(tmp);
+}
 #endif
 
 #ifdef LIKWID_WITH_ROCMON
@@ -340,4 +375,21 @@ likwid_rocmmarkerresetregion_(char* regionTag, int len)
     rocmon_markerResetRegion( tmp);
     free(tmp);
 }
+
+int __attribute__ ((visibility ("default") ))
+likwid_rocmmarkerwritefile_(char* filename, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char));
+    strncpy(tmp, filename, len * sizeof(char) );
+
+    for (int i=(len-1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i+1] = 0;
+            break;
+        }
+    }
+    rocmon_markerWriteFile(tmp);
+    free(tmp);
+}
 #endif /* LIKWID_WITH_ROCMON */
