@@ -6,13 +6,13 @@
  *      Description:  Main implementation of the performance monitoring module
  *                    for NVIDIA GPUs
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Thomas Gruber (tg), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -74,7 +74,6 @@ int likwid_nvmon_verbosity = DEBUGLEV_ONLY_ERROR;
 #include <nvmon_nvml.h>
 #include <nvmon_perfworks.h>
 
-
 LikwidNvResults* gMarkerResults = NULL;
 int gMarkerRegions = 0;
 
@@ -118,7 +117,7 @@ nvmon_init(int nrGpus, const int* gpuIds)
 
     init_configuration();
 
-    nvGroupSet = (NvmonGroupSet*) malloc(sizeof(NvmonGroupSet));
+    nvGroupSet = calloc(1, sizeof(NvmonGroupSet));
     if (nvGroupSet == NULL)
     {
         ERROR_PLAIN_PRINT(Cannot allocate group descriptor);
@@ -199,18 +198,17 @@ nvmon_init(int nrGpus, const int* gpuIds)
         }
 
         NvmonFunctions* funcs = nvGroupSet->backends[device->backend];
-        if (funcs)
+        if (funcs && funcs->createDevice)
         {
-            if (funcs->createDevice)
-                ret = funcs->createDevice(device->deviceId, device);
-        }
-        if (ret < 0)
-        {
-            ERROR_PRINT(Cannot create device %d, device->deviceId);
-            free(nvGroupSet->gpus);
-            free(nvGroupSet);
-            nvGroupSet = NULL;
-            return -ENOMEM;
+            ret = funcs->createDevice(device->deviceId, device);
+            if (ret < 0)
+            {
+                ERROR_PRINT(Cannot create device %d (error: %d), device->deviceId, ret);
+                free(nvGroupSet->gpus);
+                free(nvGroupSet);
+                nvGroupSet = NULL;
+                return -ENOMEM;
+            }
         }
     }
 
@@ -327,7 +325,6 @@ nvmon_getEventsOfGpu(int gpuId, NvmonEventList_t* list)
         return -EINVAL;
     }
 
-    //err = nvmon_cupti_functions.getEventList(available, list);
     if (gtopo->devices[available].ccapMajor < 7)
     {
         if (nvmon_cupti_functions.getEventList)
@@ -448,7 +445,6 @@ nvmon_initEventSourceLookupMaps(int gid, int gpuId)
     GroupInfo* group = &nvGroupSet->groups[gid];
 
     // Allocate memory
-    NvmonGroupSourceInfo* info;
     if (gid >= nvGroupSet->numGroupSources)
     {
         int* tmpSourceTypes = (int*) malloc(group->nevents * sizeof(int));
@@ -473,7 +469,6 @@ nvmon_initEventSourceLookupMaps(int gid, int gpuId)
             return -ENOMEM;
         }
 
-        info = tmpInfo;
         nvGroupSet->groupSources = tmpInfo;
         nvGroupSet->groupSources[gid].numEvents = group->nevents;
         nvGroupSet->groupSources[gid].sourceTypes = tmpSourceTypes;
@@ -522,7 +517,6 @@ int
 nvmon_addEventSet(const char* eventCString)
 {
     int i = 0, j = 0, k = 0, l = 0, m = 0, err = 0;
-    int isPerfGroup = 0;
 
     int devId = 0;
     int configuredEvents = 0;
@@ -618,7 +612,6 @@ nvmon_addEventSet(const char* eventCString)
             ERROR_PRINT(Cannot read performance group %s, eventCString);
             return err;
         }
-        isPerfGroup = 1;
     }
 
     // Build source lookup
@@ -1138,6 +1131,7 @@ int nvmon_getGroups(int gpuId, char*** groups, char*** shortinfos, char*** longi
 int nvmon_returnGroups(int nrgroups, char** groups, char** shortinfos, char** longinfos)
 {
     perfgroup_returnGroups(nrgroups, groups, shortinfos, longinfos);
+    return 0;
 }
 
 int nvmon_getNumberOfMetrics(int groupId)
@@ -1353,7 +1347,7 @@ nvmon_readMarkerFile(const char* filename)
         return -ENOMEM;
     }
     gMarkerRegions = regions;
-    for ( uint32_t i=0; i < regions; i++ )
+    for (int i = 0; i < regions; i++)
     {
         regionGPUs[i] = 0;
         gMarkerResults[i].gpuCount = gpus;
@@ -1448,7 +1442,7 @@ nvmon_readMarkerFile(const char* filename)
             }
         }
     }
-    for ( uint32_t i=0; i < regions; i++ )
+    for (int i = 0; i < regions; i++)
     {
         gMarkerResults[i].gpuCount = regionGPUs[i];
     }
