@@ -6,13 +6,13 @@
  *      Description:  NVML implementation of the performance monitoring module
  *                    for NVIDIA GPUs
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Thomas Gruber (tg), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -397,9 +397,12 @@ _nvml_linkLibraries()
     // Load NVML libary and link functions
     GPUDEBUG_PRINT(DEBUGLEV_DEVELOP, Init NVML Libaries);
     dl_nvml = dlopen("libnvidia-ml.so", RTLD_NOW | RTLD_GLOBAL);
-    if (!dl_nvml || dlerror() != NULL)
+    if (!dl_nvml)
+        dl_nvml = dlopen("libnvidia-ml.so.1", RTLD_NOW | RTLD_GLOBAL);
+
+    if (!dl_nvml)
     {
-        fprintf(stderr, "NVML library libnvidia-ml.so not found.");
+        fprintf(stderr, "Unable to open NVML library libnvidia-ml.so: %s", dlerror());
         return -1;
     }
 
@@ -1379,4 +1382,4 @@ nvml_getTimeToLastReadOfGroup(int groupId)
 
     // Return time as seconds
     return time*1E-9;
-}
\ No newline at end of file
+}
