@@ -5,14 +5,14 @@
  *
  *      Description:  Module implementing Intel RAPL interface
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -141,6 +141,8 @@ power_init(int cpuId)
                 case ICELAKEX1:
                 case ICELAKEX2:
                 case SAPPHIRERAPIDS:
+                case GRANITERAPIDS:
+                case SIERRAFORREST:
                     core_limits = 1;
                     power_info.hasRAPL = 1;
                     numDomains = NUM_POWER_DOMAINS;
@@ -213,7 +215,10 @@ power_init(int cpuId)
                     }
                     break;
                 case ZEN4_RYZEN:
+                case ZEN4_RYZEN2:
                 case ZEN4_EPYC:
+		case ZEN4_EPYC_BERGAMO:
+                case ZEN4_RYZEN_PRO:
                     cpuid_info.turbo = 0;
                     power_info.hasRAPL = 1;
                     power_info.statusRegWidth = 64;
@@ -236,7 +241,11 @@ power_init(int cpuId)
             break;
     }
 
-    perfmon_init_maps();
+    err = perfmon_init_maps();
+    if (err != 0)
+    {
+        return err;
+    }
     if (!HPMinitialized())
     {
         HPMinit();
@@ -398,12 +407,17 @@ power_init(int cpuId)
                 (cpuid_info.model == SKYLAKEX) ||
                 (cpuid_info.model == ICELAKEX1) ||
                 (cpuid_info.model == ICELAKEX2) ||
-                (cpuid_info.model == SAPPHIRERAPIDS) ||
                 (cpuid_info.model == XEON_PHI_KNL) ||
                 (cpuid_info.model == XEON_PHI_KML)))
             {
                 power_info.domains[DRAM].energyUnit = 15.3E-6;
             }
+            else if (cpuid_info.family == P6_FAMILY && ((cpuid_info.model == SAPPHIRERAPIDS) ||
+                     (cpuid_info.model == GRANITERAPIDS) ||
+                     (cpuid_info.model == SIERRAFORREST)))
+            {
+                power_info.domains[DRAM].energyUnit = 61E-6;
+            }
 
             for(i = 0; i < numDomains; i++)
             {
@@ -501,7 +515,7 @@ power_init(int cpuId)
             }
         }
 
-        if (cpuid_info.family == ZEN3_FAMILY && (cpuid_info.model == ZEN4_RYZEN || cpuid_info.model == ZEN4_EPYC))
+        if (cpuid_info.family == ZEN3_FAMILY && (cpuid_info.model == ZEN4_RYZEN || cpuid_info.model == ZEN4_RYZEN2 || cpuid_info.model == ZEN4_EPYC || cpuid_info.model == ZEN4_EPYC_BERGAMO))
         {
             err = HPMread(cpuId, MSR_DEV, MSR_AMD19_RAPL_L3_UNIT, &flags);
             if (err == 0)
