@@ -6,13 +6,13 @@
  *      Description:  Main implementation of the performance monitoring module
  *                    for AMD GPUs
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Author:   Thomas Gruber (tg), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -47,6 +47,10 @@
 
 #include <likwid.h>
 #include <rocmon_types.h>
+#include <amd_smi/amdsmi.h>
+#if AMDSMI_LIB_VERSION_YEAR == 23 && AMDSMI_LIB_VERSION_MAJOR == 4 && AMDSMI_LIB_VERSION_MINOR == 0 && AMDSMI_LIB_VERSION_RELEASE == 0
+typedef struct metrics_table_header_t metrics_table_header_t;
+#endif
 #include <rocm_smi/rocm_smi.h>
 
 // #include <hsa.h>
