@@ -1,17 +1,17 @@
 /*
  * =======================================================================================
  *
- *      Filename:  libnvctr.c
+ *      Filename:  rocmon_marker.c
  *
  *      Description:  Marker API interface of module rocmon
  *
- *      Version:   5.3
- *      Released:  10.11.2023
+ *      Version:   5.4.1
+ *      Released:  09.12.2024
  *
  *      Authors:  Thomas Gruber (tg), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2023 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -120,7 +120,7 @@ _rocmon_parse_gpustr(char* gpuStr, int* numGpus, int** gpuIds)
 }
 
 static void
-_rocmon_saveToFile(void)
+_rocmon_saveToFile(const char* markerfile)
 {
     /* File format
      * 1 numberOfGPUs numberOfRegions numberOfGpuGroups
@@ -130,14 +130,6 @@ _rocmon_saveToFile(void)
      * 5 regionID groupID gpuID callCount timeActive numEvents countersvalues(space separated)
      */
 
-    // Get markerfile path from environment
-    char* markerfile = getenv("LIKWID_ROCMON_FILEPATH");
-    if (markerfile == NULL)
-    {
-        fprintf(stderr, "Is the application executed with LIKWID wrapper? No file path for the Rocmon Marker API output defined.\n");
-        return;
-    }
-
     // Verify there is something to output
     int numberOfRegions = get_map_size(gpu_maps[0]);
     int numberOfGPUs = rocmon_getNumberOfGPUs();
@@ -245,6 +237,7 @@ rocmon_markerInit(void)
     char* gpuStr = getenv("LIKWID_ROCMON_GPUS");
     char* gpuFileStr = getenv("LIKWID_ROCMON_FILEPATH");
     char* verbosityStr = getenv("LIKWID_ROCMON_VERBOSITY");
+    char* debugStr = getenv("LIKWID_DEBUG");
 
     // Validate environment variables are set
     if ((eventStr == NULL) || (gpuStr == NULL) || (gpuFileStr == NULL))
@@ -256,6 +249,11 @@ rocmon_markerInit(void)
         int v = atoi(verbosityStr);
         rocmon_setVerbosity(v);
     }
+    if (debugStr != NULL)
+    {
+        int v = atoi(debugStr);
+        perfmon_setVerbosity(v);
+    }
 
     // Init timer module
     timer_init();
@@ -379,11 +377,33 @@ rocmon_markerClose(void)
     // Stop counters
     rocmon_stopCounters();
 
-    _rocmon_saveToFile();
+    // Get markerfile path from environment
+    char* markerfile = getenv("LIKWID_ROCMON_FILEPATH");
+    if (markerfile == NULL)
+    {
+        fprintf(stderr, "Is the application executed with LIKWID wrapper? No file path for the Rocmon Marker API output defined.\n");
+        return;
+    }
+    else
+    {
+        _rocmon_saveToFile(markerfile);
+    }
+
     _rocmon_finalize();
 }
 
 
+int
+rocmon_markerWriteFile(const char* markerfile)
+{
+    if (!markerfile)
+    {
+        return -EINVAL;
+    }
+    _rocmon_saveToFile(markerfile);
+    return 0;
+}
+
 int
 rocmon_markerRegisterRegion(const char* regionTag)
 {
