@@ -13,6 +13,18 @@ for L in $(sinfo -t idle -h --partition=work -o "%n %t" | grep "idle" | cut -d '
         arch="arm8"
         depend="build-arm8-perf"
     fi
+    if [ "$L" = "lukewarm" ]; then
+        arch="arm8"
+        depend="build-arm8-perf"
+    fi
+    if [ "$L" = "gracehop1" ]; then
+        arch="arm8"
+        depend="build-arm8-perf"
+    fi
+    if [ "$L" = "gracesup1" ]; then
+        arch="arm8"
+        depend="build-arm8-perf"
+    fi
 
     cat <<EOF
 test-$L-perf:
@@ -26,6 +38,7 @@ test-$L-perf:
   tags:
     - testcluster
   before_script:
+    - rm -rf /tmp/$depend
     - cp -r $depend /tmp/$depend
     - cd /tmp/$depend
     - export PATH=/tmp/$depend/bin:\$PATH
@@ -34,6 +47,7 @@ test-$L-perf:
     - likwid-topology
     - likwid-pin -p
     - likwid-perfctr -i
+    - likwid-sysfeatures -a
   after_script:
     - rm -rf /tmp/$depend
 EOF
@@ -59,6 +73,7 @@ test-$L-daemon:
   tags:
     - testcluster
   before_script:
+    - rm -rf /tmp/$depend
     - cp -r $depend /tmp/$depend
     - cd /tmp/$depend
     - export PATH=/tmp/$depend/bin:\$PATH
@@ -67,6 +82,7 @@ test-$L-daemon:
     - likwid-topology
     - likwid-pin -p
     - likwid-perfctr -i
+    - likwid-sysfeatures -a
   after_script:
     - rm -rf /tmp/$depend
 EOF
